from django.conf import settings
from django.contrib.auth.models import Group
from django.contrib.sites.models import Site
from django.db import models
from django.template import Context, Template
from django.utils.translation import gettext_lazy as _
from rdmo.core.models import TranslationMixin
from rdmo.core.utils import copy_model, get_pandoc_main_version, join_url
from rdmo.questions.models import Catalog

from .managers import ViewManager
from .utils import ProjectWrapper


class View(models.Model, TranslationMixin):

    objects = ViewManager()

    uri = models.URLField(
        max_length=640, blank=True,
        verbose_name=_('URI'),
        help_text=_('The Uniform Resource Identifier of this view (auto-generated).')
    )
    uri_prefix = models.URLField(
        max_length=256,
        verbose_name=_('URI Prefix'),
        help_text=_('The prefix for the URI of this view.')
    )
    key = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Key'),
        help_text=_('The internal identifier of this view.')
    )
    comment = models.TextField(
        blank=True,
        verbose_name=_('Comment'),
        help_text=_('Additional internal information about this view.')
    )
    locked = models.BooleanField(
        default=False,
        verbose_name=_('Locked'),
        help_text=_('Designates whether this view can be changed.')
    )
    catalogs = models.ManyToManyField(
        Catalog, blank=True,
        verbose_name=_('Catalogs'),
        help_text=_('The catalogs this view can be used with. An empty list implies that this view can be used with every catalog.')
    )
    sites = models.ManyToManyField(
        Site, blank=True,
        verbose_name=_('Sites'),
        help_text=_('The sites this view belongs to (in a multi site setup).')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Group'),
        help_text=_('The groups for which this view is active.')
    )
    template = models.TextField(
        blank=True,
        verbose_name=_('Template'),
        help_text=_('The template for this view, written in Django template language.')
    )
    title_lang1 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (primary)'),
        help_text=_('The title for this view in the primary language.')
    )
    title_lang2 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (secondary)'),
        help_text=_('The title for this view in the secondary language.')
    )
    title_lang3 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (tertiary)'),
        help_text=_('The title for this view in the tertiary language.')
    )
    title_lang4 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (quaternary)'),
        help_text=_('The title for this view in the quaternary language.')
    )
    title_lang5 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (quinary)'),
        help_text=_('The title for this view in the quinary language.')
    )
    help_lang1 = models.TextField(
        blank=True,
        verbose_name=_('Help (primary)'),
        help_text=_('The help text for this view in the primary language.')
    )
    help_lang2 = models.TextField(
        blank=True,
        verbose_name=_('Help (secondary)'),
        help_text=_('The help text for this view in the secondary language.')
    )
    help_lang3 = models.TextField(
        blank=True,
        verbose_name=_('Help (tertiary)'),
        help_text=_('The help text for this view in the tertiary language.')
    )
    help_lang4 = models.TextField(
        blank=True,
        verbose_name=_('Help (quaternary)'),
        help_text=_('The help text for this view in the quaternary language.')
    )
    help_lang5 = models.TextField(
        blank=True,
        verbose_name=_('Help (quinary)'),
        help_text=_('The help text for this view in the quinary language.')
    )
    available = models.BooleanField(
        default=True,
        verbose_name=_('Available'),
        help_text=_('Designates whether this view is generally available for projects.')
    )

    class Meta:
        ordering = ('key', )
        verbose_name = _('View')
        verbose_name_plural = _('Views')

    def __str__(self):
        return self.key

    def save(self, *args, **kwargs):
        self.uri = self.build_uri(self.uri_prefix, self.key)
        super().save(*args, **kwargs)

    def copy(self, uri_prefix, key):
        view = copy_model(self, uri_prefix=uri_prefix, key=key)

        # copy m2m fields
        view.catalogs.set(self.catalogs.all())
        view.sites.set(self.sites.all())
        view.groups.set(self.groups.all())

        return view

    @property
    def title(self):
        return self.trans('title')

    @property
    def help(self):
        return self.trans('help')

    @property
    def is_locked(self):
        return self.locked

    def render(self, project, snapshot=None, export_format=None):
        # render the template to a html string
        # it is important not to use models here
        project_wrapper = ProjectWrapper(project, snapshot)
        return Template(self.template).render(Context({
            'project': project_wrapper,
            'conditions': project_wrapper.conditions,
            'format': export_format,
            'pandoc_version': get_pandoc_main_version()
        }))

    @classmethod
    def build_uri(cls, uri_prefix, key):
        assert key
        return join_url(uri_prefix or settings.DEFAULT_URI_PREFIX, '/views/', key)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-04 14:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0030_permissions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='unit',
            field=models.CharField(blank=True, help_text='Unit for this attribute.', max_length=64, null=True, verbose_name='Unit'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='value_type',
            field=models.CharField(choices=[('text', 'Text'), ('url', 'URL'), ('integer', 'Integer'), ('float', 'Float'), ('boolean', 'Boolean'), ('datetime', 'Datetime'), ('options', 'Options')], help_text='Type of value for this attribute.', max_length=8, verbose_name='Value type'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this attribute/entity.', max_length=128, null=True, verbose_name='Key'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-05-22 10:04
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('accounts', '0009_proxyuser'),
    ]

    operations = [
        migrations.CreateModel(
            name='ConsentFieldValue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('consent', models.BooleanField(default=False, help_text='Designates whether the user has agreed to the terms of use.', verbose_name='Consent')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('user',),
                'verbose_name': 'Consent field value',
                'verbose_name_plural': 'Consent field values',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def set_questionentity_is_collection(apps, schema_editor):
    QuestionEntity = apps.get_model('questions', 'QuestionEntity')

    for queryset in QuestionEntity.objects.filter(question=None):

        try:
            if queryset.attribute_entity.parent_collection:
                queryset.attribute_entity = queryset.attribute_entity.parent_collection
                queryset.is_collection = True
            else:
                queryset.is_collection = queryset.attribute_entity.is_collection
        except AttributeError:
            queryset.is_collection = False

        queryset.save()

    for question in QuestionEntity.objects.exclude(question=None):

        try:
            question.is_collection = question.attribute_entity.is_collection
        except AttributeError:
            queryset.is_collection = False

        question.save()


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0021_questionentity_is_collection'),
    ]

    operations = [
        migrations.RunPython(set_questionentity_is_collection),
    ]



from django.core.exceptions import (MultipleObjectsReturned,
                                    ObjectDoesNotExist, ValidationError)
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers


class InstanceValidator(object):

    '''
    BaseValidator which should work with model instances, used by

    1) the DRF serializer (having self.serializer)
    2) the form clean method through the admin interface

    It is used as InstanceValidator() for (1) and InstanceValidator(instance)(self.cleaned_data) for (2)
    '''

    def __init__(self, instance=None):
        self.instance = instance
        self.serializer = None

    def set_context(self, serializer):
        self.instance = serializer.instance
        self.serializer = serializer

    def raise_validation_error(self, errors):
        if self.serializer:
            raise serializers.ValidationError(errors)
        else:
            raise ValidationError(errors)

    def __call__(self, value):
        raise NotImplementedError


class UniqueURIValidator(InstanceValidator):

    model = None

    def __call__(self, data):
        self.validate(self.model, self.instance, self.get_uri(data))

    def validate(self, model, instance, uri):
        try:
            if instance:
                model.objects.exclude(pk=instance.id).get(uri=uri)
            else:
                model.objects.get(uri=uri)
        except MultipleObjectsReturned:
            pass
        except ObjectDoesNotExist:
            return

        self.raise_validation_error({
            'key': _('%(model)s with the uri "%(uri)s" already exists.') % {
                'model': model._meta.verbose_name.title(),
                'uri': uri
            }
        })

    def get_uri(self, data):
        raise NotImplementedError


class LockedValidator(InstanceValidator):

    parent_field = None

    def __call__(self, data):
        is_locked = False

        # lock if a parent_field is set and a parent is set and the parent is locked
        if self.parent_field:
            parent = data.get(self.parent_field)
            if parent:
                is_locked |= parent.is_locked

        # lock only if the instance is now locked and was locked before
        if self.instance:
            is_locked |= data.get('locked', False) and self.instance.is_locked

        if is_locked:
            if data.get('locked'):
                raise self.raise_validation_error({
                    'locked': _('The element is locked.')
                })
            else:
                raise self.raise_validation_error({
                    'locked': _('A superior element is locked.')
                })



# Generated by Django 2.2.13 on 2020-10-02 10:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0040_issueresource'),
    ]

    operations = [
        migrations.AddField(
            model_name='value',
            name='external_id',
            field=models.CharField(blank=True, help_text='External id for this value.', max_length=256, verbose_name='External id'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-09-28 11:23
from __future__ import unicode_literals

from django.db import migrations


def set_value_unit_and_type(apps, schema_editor):
    Value = apps.get_model('projects', 'Value')

    for value in Value.objects.all():
        try:
            value.value_type = value.attribute.value_type
        except AttributeError:
            value.value_type = 'text'

        try:
            value.unit = value.attribute.unit or ''
        except AttributeError:
            value.unit = ''

        value.save()


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0017_value_unit_and_type'),
    ]

    operations = [
        migrations.RunPython(set_value_unit_and_type),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-27 15:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('views', '0005_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='view',
            name='help_de',
            field=models.TextField(blank=True, help_text='The German help text for this view.', null=True, verbose_name='Help (de)'),
        ),
        migrations.AddField(
            model_name='view',
            name='help_en',
            field=models.TextField(blank=True, help_text='The English help text for this view.', null=True, verbose_name='Help (en)'),
        ),
        migrations.AddField(
            model_name='view',
            name='title_de',
            field=models.CharField(blank=True, help_text='The German title for this view.', max_length=256, null=True, verbose_name='Title (de)'),
        ),
        migrations.AddField(
            model_name='view',
            name='title_en',
            field=models.CharField(blank=True, help_text='The English title for this view.', max_length=256, null=True, verbose_name='Title (en)'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-01-30 14:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0038_rename_de_to_lang2'),
    ]

    operations = [
        migrations.AlterField(
            model_name='catalog',
            name='title_lang1',
            field=models.CharField(help_text='The title for this catalog in the primary language.', max_length=256, verbose_name='Title (primary)'),
        ),
        migrations.AlterField(
            model_name='catalog',
            name='title_lang2',
            field=models.CharField(help_text='The title for this catalog in the secondary language.', max_length=256, verbose_name='Title (secondary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='help_lang1',
            field=models.TextField(blank=True, help_text='The help text for this question in the primary language.', null=True, verbose_name='Help (primary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='help_lang2',
            field=models.TextField(blank=True, help_text='The help text for this question in the secondary language.', null=True, verbose_name='Help (secondary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='text_lang1',
            field=models.TextField(help_text='The text for this question in the primary language.', verbose_name='Text (primary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='text_lang2',
            field=models.TextField(help_text='The text for this question in the secondary language.', verbose_name='Text (secondary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='verbose_name_lang1',
            field=models.CharField(blank=True, help_text='The name displayed for this question in the primary language.', max_length=256, verbose_name='Name (primary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='verbose_name_lang2',
            field=models.CharField(blank=True, help_text='The name displayed for this question in the secondary language.', max_length=256, verbose_name='Name (secondary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='verbose_name_plural_lang1',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the primary language.', max_length=256, verbose_name='Plural name (primary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='verbose_name_plural_lang2',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the secondary language.', max_length=256, verbose_name='Plural name (secondary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='help_lang1',
            field=models.TextField(blank=True, help_text='The help text for this questionset in the primary language.', null=True, verbose_name='Help (primary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='help_lang2',
            field=models.TextField(blank=True, help_text='The help text for this questionset in the secondary language.', null=True, verbose_name='Help (secondary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='title_lang1',
            field=models.CharField(help_text='The title for this questionset in the primary language.', max_length=256, verbose_name='Title (primary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='title_lang2',
            field=models.CharField(help_text='The title for this questionset in the secondary language.', max_length=256, verbose_name='Title (secondary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_lang1',
            field=models.CharField(blank=True, help_text='The name displayed for this question in the primary language.', max_length=256, verbose_name='Name (primary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_lang2',
            field=models.CharField(blank=True, help_text='The  name displayed for this question in the secondary language.', max_length=256, verbose_name='Name (secondary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_plural_lang1',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the primary language.', max_length=256, verbose_name='Plural name (primary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_plural_lang2',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the secondary language.', max_length=256, verbose_name='Plural name (secondary)'),
        ),
        migrations.AlterField(
            model_name='section',
            name='title_lang1',
            field=models.CharField(help_text='The title for this section in the primary language.', max_length=256, verbose_name='Title (primary)'),
        ),
        migrations.AlterField(
            model_name='section',
            name='title_lang2',
            field=models.CharField(help_text='The title for this section in the secondary language.', max_length=256, verbose_name='Title (secondary)'),
        ),
    ]



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from rdmo.core.models import Model, TranslationMixin
from rdmo.core.utils import copy_model, join_url

from .catalog import Catalog


class Section(Model, TranslationMixin):

    uri = models.URLField(
        max_length=640, blank=True,
        verbose_name=_('URI'),
        help_text=_('The Uniform Resource Identifier of this section (auto-generated).')
    )
    uri_prefix = models.URLField(
        max_length=256,
        verbose_name=_('URI Prefix'),
        help_text=_('The prefix for the URI of this section.')
    )
    key = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Key'),
        help_text=_('The internal identifier of this section.')
    )
    path = models.CharField(
        max_length=512, blank=True,
        verbose_name=_('Label'),
        help_text=_('The path part of the URI of this section (auto-generated).')
    )
    comment = models.TextField(
        blank=True,
        verbose_name=_('Comment'),
        help_text=_('Additional internal information about this section.')
    )
    locked = models.BooleanField(
        default=False,
        verbose_name=_('Locked'),
        help_text=_('Designates whether this section (and it\'s question sets and questions) can be changed.')
    )
    catalog = models.ForeignKey(
        Catalog, on_delete=models.CASCADE, related_name='sections',
        verbose_name=_('Catalog'),
        help_text=_('The catalog this section belongs to.')
    )
    order = models.IntegerField(
        default=0,
        verbose_name=_('Order'),
        help_text=_('Position in lists.')
    )
    title_lang1 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (primary)'),
        help_text=_('The title for this section in the primary language.')
    )
    title_lang2 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (secondary)'),
        help_text=_('The title for this section in the secondary language.')
    )
    title_lang3 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (tertiary)'),
        help_text=_('The title for this section in the tertiary language.')
    )
    title_lang4 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (quaternary)'),
        help_text=_('The title for this section in the quaternary language.')
    )
    title_lang5 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (quinary)'),
        help_text=_('The title for this section in the quinary language.')
    )

    class Meta:
        ordering = ('catalog__order', 'order')
        verbose_name = _('Section')
        verbose_name_plural = _('Sections')

    def __str__(self):
        return self.path

    def save(self, *args, **kwargs):
        self.path = self.build_path(self.key, self.catalog)
        self.uri = self.build_uri(self.uri_prefix, self.path)

        super().save(*args, **kwargs)

        for questionsets in self.questionsets.all():
            questionsets.save()

    def copy(self, uri_prefix, key, catalog=None):
        section = copy_model(self, uri_prefix=uri_prefix, key=key, catalog=catalog or self.catalog)

        # copy children
        for questionset in self.questionsets.filter(questionset=None):
            questionset.copy(uri_prefix, questionset.key, section=section)

        return section

    @property
    def parent_fields(self):
        return ('catalog', )

    @property
    def title(self):
        return self.trans('title')

    @property
    def is_locked(self):
        return self.locked or self.catalog.is_locked

    @classmethod
    def build_path(cls, key, catalog):
        assert key
        assert catalog
        return catalog.key + '/' + key

    @classmethod
    def build_uri(cls, uri_prefix, path):
        assert path
        return join_url(uri_prefix or settings.DEFAULT_URI_PREFIX, '/questions/', path)



from django import forms
from django.contrib import admin
from django.db import models

from .models import Attribute
from .validators import (AttributeLockedValidator, AttributeParentValidator,
                         AttributeUniqueURIValidator)


class AttributeAdminForm(forms.ModelForm):
    key = forms.SlugField(required=True)

    class Meta:
        model = Attribute
        fields = '__all__'

    def clean(self):
        AttributeUniqueURIValidator(self.instance)(self.cleaned_data)
        AttributeParentValidator(self.instance)(self.cleaned_data)
        AttributeLockedValidator(self.instance)(self.cleaned_data)


class AttributeAdmin(admin.ModelAdmin):
    form = AttributeAdminForm

    list_display = ('uri', 'projects_count', 'values_count')
    search_fields = ('uri', )
    readonly_fields = ('uri', 'path', 'projects_count', 'values_count')

    def get_queryset(self, request):
        return super().get_queryset(request) \
                      .annotate(values_count=models.Count('values')) \
                      .annotate(projects_count=models.Count('values__project', distinct=True))

    def values_count(self, obj):
        return obj.values_count

    def projects_count(self, obj):
        return obj.projects_count


admin.site.register(Attribute, AttributeAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-03 13:10
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0010_label'),
    ]

    operations = [
        migrations.RenameField(
            model_name='questionentity',
            old_name='label',
            new_name='path',
        ),
        migrations.RenameField(
            model_name='section',
            old_name='label',
            new_name='path',
        ),
        migrations.RenameField(
            model_name='subsection',
            old_name='label',
            new_name='path',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-09-28 11:23
from __future__ import unicode_literals

from django.db import migrations


def set_project(apps, schema_editor):
    Value = apps.get_model('projects', 'Value')
    Snapshot = apps.get_model('projects', 'Snapshot')
    for value in Value.objects.all():
        value.project = value.snapshot.project
        value.snapshot = None
        value.save()

    for snapshot in Snapshot.objects.all():
        snapshot.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0006_project_values'),
    ]

    operations = [
        migrations.RunPython(set_project),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-11 13:24
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0006_db_constraint_false'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='condition',
            options={'ordering': ('title',), 'verbose_name': 'Condition', 'verbose_name_plural': 'Conditions'},
        ),
    ]



from django.urls import include, path
from rest_framework import routers

from ..viewsets import OverlayViewSet

app_name = 'v1-overlays'

router = routers.DefaultRouter()
router.register(r'overlays', OverlayViewSet, basename='overlay')

urlpatterns = [
    path('', include(router.urls)),
]



import re

from django.http import HttpResponse

from rdmo.core.exports import prettify_xml
from rdmo.core.plugins import Plugin
from rdmo.core.utils import render_to_csv
from rdmo.questions.models import Question

from .renderers import XMLRenderer
from .serializers.export import ProjectSerializer as ExportSerializer


class Export(Plugin):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.project = None
        self.snapshot = None

    def render(self):
        raise NotImplementedError

    def get_set(self, path, set_prefix=''):
        return self.project.values.filter(snapshot=self.snapshot, attribute__path=path, set_prefix=set_prefix) \
                                  .order_by('set_index', 'collection_index')

    def get_values(self, path, set_prefix='', set_index=0):
        return self.project.values.filter(snapshot=self.snapshot, attribute__path=path, set_prefix=set_prefix, set_index=set_index) \
                                  .order_by('collection_index')

    def get_value(self, path, set_prefix='', set_index=0, collection_index=0):
        try:
            return self.get_values(path, set_prefix=set_prefix, set_index=set_index)[collection_index]
        except IndexError:
            return None

    def get_text(self, path, set_prefix='', set_index=0, collection_index=0):
        try:
            return self.get_values(path, set_prefix=set_prefix, set_index=set_index)[collection_index].text
        except IndexError:
            return None

    def get_timestamp(self, path, set_prefix='', set_index=0, collection_index=0):
        try:
            return self.get_values(path, set_prefix=set_prefix, set_index=set_index)[collection_index].value.isoformat()
        except (IndexError, AttributeError):
            return None

    def get_year(self, path, set_prefix='', set_index=0, collection_index=0):
        try:
            return self.get_values(path, set_prefix=set_prefix, set_index=set_index)[collection_index].value.year
        except (IndexError, AttributeError):
            return None

    def get_list(self, path, set_prefix='', set_index=0):
        values = self.get_values(path, set_prefix=set_prefix, set_index=set_index)
        return [value.text for value in values if value.text]

    def get_bool(self, path, set_prefix='', set_index=0, collection_index=0):
        value = self.get_value(path, set_prefix=set_prefix, set_index=set_index, collection_index=collection_index)
        if value:
            return True if value.text == '1' else False
        else:
            return None

    def get_option(self, options, path, set_prefix='', set_index=0, collection_index=0, default=None):
        value = self.get_value(path, set_prefix=set_prefix, set_index=set_index, collection_index=collection_index)
        if value and value.option:
            # lookup option dict in class
            return options.get(value.option.path, default)
        else:
            return default


class CSVExport(Export):

    delimiter = ','

    def render(self):
        queryset = self.project.values.filter(snapshot=None)
        data = []

        for question in Question.objects.order_by_catalog(self.project.catalog):
            if question.questionset.is_collection and question.questionset.attribute:
                if question.questionset.attribute.uri.endswith('/id'):
                    set_attribute_uri = question.questionset.attribute
                else:
                    set_attribute_uri = question.questionset.attribute.uri.rstrip('/') + '/id'

                for value_set in queryset.filter(attribute__uri=set_attribute_uri):
                    values = queryset.filter(attribute=question.attribute, set_index=value_set.set_index) \
                                     .order_by('set_prefix', 'set_index', 'collection_index')
                    data.append((self.stringify(question.text), self.stringify(value_set.value), self.stringify_values(values)))
            else:
                values = queryset.filter(attribute=question.attribute).order_by('set_prefix', 'set_index', 'collection_index')

                data.append((self.stringify(question.text), '', self.stringify_values(values)))

        return render_to_csv(self.project.title, data, self.delimiter)

    def stringify_values(self, values):
        if values is not None:
            return '; '.join([self.stringify(value.value_and_unit) for value in values])
        else:
            return ''

    def stringify(self, el):
        if el is None:
            return ''
        else:
            return re.sub(r'\s+', ' ', str(el))


class CSVCommaExport(CSVExport):
    delimiter = ','


class CSVSemicolonExport(CSVExport):
    delimiter = ';'


class RDMOXMLExport(Export):

    def render(self):
        serializer = ExportSerializer(self.project)
        xmldata = XMLRenderer().render(serializer.data)
        response = HttpResponse(prettify_xml(xmldata), content_type="application/xml")
        response['Content-Disposition'] = 'filename="%s.xml"' % self.project.title
        return response



from rest_framework.serializers import ValidationError

from rdmo.core.plugins import get_plugin


class ProviderValidator():

    def __call__(self, data):
        provider_key = data.get('provider_key')
        provider = get_plugin('SERVICE_PROVIDERS', provider_key)
        if provider is None:
            raise ValidationError({
                'provider_key': 'Please provide a valid provider.'
            })

        try:
            options = {option.get('key'): option.get('value') for option in data.get('options', [])}
        except KeyError:
            raise ValidationError({
                'options': 'Options need to be of the form "{"key": "": "value": ""}".'
            })

        for key in options:
            if key not in [field.get('key') for field in provider.fields]:
                raise ValidationError({
                    'options': 'Key "{}" is not valid.'.format(key)
                })

        for field in provider.fields:
            if field.get('required', True) and field.get('key') not in options:
                raise ValidationError({
                    'options': 'Key "{}" is required.'.format(field.get('key'))
                })



import logging

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.views.generic import ListView, TemplateView
from rdmo.core.exports import XMLResponse
from rdmo.core.utils import (get_model_field_meta, render_to_csv,
                             render_to_format)
from rdmo.core.views import CSRFViewMixin, ModelPermissionMixin

from .models import Attribute
from .renderers import AttributeRenderer
from .serializers.export import AttributeExportSerializer

logger = logging.getLogger(__name__)


class DomainView(ModelPermissionMixin, CSRFViewMixin, TemplateView):
    template_name = 'domain/domain.html'
    permission_required = 'domain.view_attribute'

    def get_context_data(self, **kwargs):
        context = super(DomainView, self).get_context_data(**kwargs)
        context['export_formats'] = settings.EXPORT_FORMATS
        context['meta'] = {
            'Attribute': get_model_field_meta(Attribute)
        }
        return context


class DomainExportView(ModelPermissionMixin, ListView):
    model = Attribute
    context_object_name = 'attributes'
    permission_required = 'domain.view_attribute'

    def render_to_response(self, context, **response_kwargs):
        format = self.kwargs.get('format')
        if format == 'xml':
            serializer = AttributeExportSerializer(context['attributes'], many=True)
            xml = AttributeRenderer().render(serializer.data)
            return XMLResponse(xml, name='domain')
        elif format[:3] == 'csv':
            if format == 'csvcomma':
                delimiter = ','
            else:
                delimiter = ';'
            rows = []
            for attribute in context['attributes']:
                rows.append((
                    attribute.key,
                    attribute.comment,
                    attribute.uri
                ))
            return render_to_csv(_('Domain'), rows, delimiter)
        else:
            return render_to_format(self.request, format, _('Domain'), 'domain/domain_export.html', context)



from django.db import models
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet

from rdmo.core.exports import XMLResponse
from rdmo.core.permissions import HasModelPermission
from rdmo.core.viewsets import CopyModelMixin

from .models import Attribute
from .renderers import AttributeRenderer
from .serializers.export import AttributeExportSerializer
from .serializers.v1 import (AttributeIndexSerializer,
                             AttributeNestedSerializer, AttributeSerializer)


class AttributeViewSet(CopyModelMixin, ModelViewSet):
    permission_classes = (HasModelPermission, )
    queryset = Attribute.objects.order_by('path') \
                        .annotate(values_count=models.Count('values')) \
                        .annotate(projects_count=models.Count('values__project', distinct=True)) \
                        .prefetch_related('conditions', 'questionsets', 'questions', 'tasks_as_start', 'tasks_as_end')

    serializer_class = AttributeSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'uri',
        'path',
        'key',
        'parent'
    )

    @action(detail=False)
    def nested(self, request):
        queryset = Attribute.objects.get_cached_trees()
        serializer = AttributeNestedSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False)
    def index(self, request):
        queryset = self.get_queryset()
        serializer = AttributeIndexSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, permission_classes=[HasModelPermission])
    def export(self, request):
        serializer = AttributeExportSerializer(self.get_queryset(), many=True)
        xml = AttributeRenderer().render(serializer.data)
        return XMLResponse(xml, name='attributes')

    @action(detail=True, url_path='export', permission_classes=[HasModelPermission])
    def detail_export(self, request, pk=None):
        queryset = self.get_object().get_descendants(include_self=True)
        serializer = AttributeExportSerializer(queryset, many=True)
        xml = AttributeRenderer().render(serializer.data)
        return XMLResponse(xml, name=self.get_object().key)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-07-18 10:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0007_db_index'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attributeentity',
            name='parent_collection',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='domain.AttributeEntity'),
        ),
    ]



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from rdmo.core.models import Model
from rdmo.questions.models import QuestionSet


class Continuation(Model):

    project = models.ForeignKey(
        'Project', on_delete=models.CASCADE, related_name='+',
        verbose_name=_('Project'),
        help_text=_('The project for this continuation.')
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='+',
        verbose_name=_('User'),
        help_text=_('The user for this continuation.')
    )
    questionset = models.ForeignKey(
        QuestionSet, on_delete=models.CASCADE, related_name='+',
        verbose_name=_('Question set'),
        help_text=_('The question set for this continuation.')
    )

    class Meta:
        ordering = ('user', 'project')
        verbose_name = _('Continuation')
        verbose_name_plural = _('Continuations')

    def __str__(self):
        return '%s/%s/%s' % (self.project, self.user, self.questionset)



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Attribute
from ..serializers.v1 import AttributeSerializer
from ..validators import AttributeUniqueURIValidator


def test_validator_create(db):
    AttributeUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'parent': Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    })


def test_validator_create_error(db):
    with pytest.raises(ValidationError):
        AttributeUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': 'text',
            'parent': Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
        })


def test_validator_update(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    AttributeUniqueURIValidator(attribute)({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent': attribute.parent
    })


def test_validator_update_error(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')

    with pytest.raises(ValidationError):
        AttributeUniqueURIValidator(attribute)({
            'uri_prefix': attribute.uri_prefix,
            'key': 'textarea',
            'parent': attribute.parent
        })


def test_validator_serializer_create(db):
    validator = AttributeUniqueURIValidator()
    validator.set_context(AttributeSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'parent': Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    })


def test_validator_serializer_create_error(db):
    validator = AttributeUniqueURIValidator()
    validator.set_context(AttributeSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': 'text',
            'parent': Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
        })


def test_validator_serializer_update(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    validator = AttributeUniqueURIValidator()
    validator.set_context(AttributeSerializer(instance=attribute))

    validator({
        'uri_prefix': attribute.uri_prefix,
        'key': 'test',
        'parent':  attribute.parent
    })


def test_validator_serializer_update_error(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    validator = AttributeUniqueURIValidator()
    validator.set_context(AttributeSerializer(instance=attribute))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': attribute.uri_prefix,
            'key': 'textarea',
            'parent': attribute.parent
        })



from django.db import models

from rdmo.core.managers import (AvailabilityManagerMixin,
                                AvailabilityQuerySetMixin,
                                CurrentSiteManagerMixin,
                                CurrentSiteQuerySetMixin, GroupsManagerMixin,
                                GroupsQuerySetMixin)


class CatalogQuestionSet(CurrentSiteQuerySetMixin, GroupsQuerySetMixin, AvailabilityQuerySetMixin, models.QuerySet):

    def filter_catalog(self, catalog):
        return self.filter(models.Q(catalogs=None) | models.Q(catalogs=catalog))


class CatalogManager(CurrentSiteManagerMixin, GroupsManagerMixin, AvailabilityManagerMixin, models.Manager):

    def get_queryset(self):
        return CatalogQuestionSet(self.model, using=self._db)

    def filter_catalog(self, catalog):
        return self.get_queryset().filter_catalog(catalog)


class QuestionSetQuerySet(models.QuerySet):

    def order_by_catalog(self, catalog):
        return self.filter(section__catalog=catalog, questionset=None).order_by('section__order', 'order')

    def filter_by_catalog(self, catalog):
        return self.filter(section__catalog=catalog)


class QuestionSetManager(models.Manager):

    def get_queryset(self):
        return QuestionSetQuerySet(self.model, using=self._db)

    def order_by_catalog(self, catalog):
        return self.get_queryset().order_by_catalog(catalog)

    def filter_by_catalog(self, catalog):
        return self.get_queryset().filter_by_catalog(catalog)


class QuestionQuerySet(models.QuerySet):

    def order_by_catalog(self, catalog):
        return self.filter(questionset__section__catalog=catalog).order_by('questionset__section__order', 'questionset__order', 'order')

    def filter_by_catalog(self, catalog):
        return self.filter(questionset__section__catalog=catalog)


class QuestionManager(models.Manager):

    def get_queryset(self):
        return QuestionQuerySet(self.model, using=self._db)

    def order_by_catalog(self, catalog):
        return self.get_queryset().order_by_catalog(catalog)

    def filter_by_catalog(self, catalog):
        return self.get_queryset().filter_by_catalog(catalog)



# Generated by Django 2.2.2 on 2019-06-17 11:10

import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('questions', '0044_require_uri_prefix'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='catalog',
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('on_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.AddField(
            model_name='catalog',
            name='sites',
            field=models.ManyToManyField(help_text='The sites this catalog belongs to (in a multi site setup).', to='sites.Site', verbose_name='Sites'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-04 14:10
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0009_timeframe'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional internal information about this task.', null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='task',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this task.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='timeframe',
            name='days_after',
            field=models.IntegerField(blank=True, help_text='Additional days after the end date.', null=True, verbose_name='Days after'),
        ),
        migrations.AlterField(
            model_name='timeframe',
            name='days_before',
            field=models.IntegerField(blank=True, help_text='Additional days before the start date.', null=True, verbose_name='Days before'),
        ),
        migrations.AlterField(
            model_name='timeframe',
            name='end_attribute',
            field=models.ForeignKey(blank=True, help_text='The Attribute that is setting the end date for this task (optional, if no end date attribute is given, the start date attribute sets also the end date).', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute', verbose_name='End date attribute'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-25 13:36

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('accounts', '0016_remove_null_true'),
    ]

    operations = [
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('manager', models.ManyToManyField(help_text='The sites for which this user is manager.', related_name='managers', to='sites.Site', verbose_name='Manager')),
                ('member', models.ManyToManyField(help_text='The sites for which this user is a member.', related_name='members', to='sites.Site', verbose_name='Member')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'Roles',
                'verbose_name': 'Role',
                'ordering': ('user',),
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-27 10:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0009_options'),
        ('options', '0003_data_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='optionset',
            name='conditions',
            field=models.ManyToManyField(blank=True, to='conditions.Condition'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-19 16:53
from __future__ import unicode_literals

from django.db import migrations
from django.utils.text import slugify


def create_identifier(apps, schema_editor):

    Task = apps.get_model('tasks', 'Task')

    for obj in Task.objects.all():
        if not obj.key:
            obj.key = slugify(obj.title_en)
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0004_refactoring'),
    ]

    operations = [
        migrations.RunPython(create_identifier),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-01 11:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0002_many_to_many_for_conditions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='condition',
            name='relation',
            field=models.CharField(choices=[('eq', 'is equal to (==)'), ('neq', 'is not equal to (!=)'), ('contains', 'contains'), ('gt', 'is greater than (>)'), ('gte', 'is greater than or equal (>=)'), ('lt', 'is lesser than (<)'), ('lte', 'is lesser than or equal (<=)')], max_length=8),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-03 12:08
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0007_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='option',
            name='path',
            field=models.SlugField(blank=True, help_text='The path part of the URI for this option.', max_length=512, null=True, verbose_name='Path'),
        ),
    ]



from rest_framework import serializers

from rdmo.core.serializers import TranslationSerializerMixin

from ..models import Catalog, Question, QuestionSet, Section


class QuestionExportSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    attribute = serializers.CharField(source='attribute.uri', default=None, read_only=True)
    questionset = serializers.CharField(source='questionset.uri', default=None, read_only=True)
    default_option = serializers.CharField(source='default_option.uri', default=None, read_only=True)
    optionsets = serializers.SerializerMethodField()
    conditions = serializers.SerializerMethodField()

    class Meta:
        model = Question
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'path',
            'comment',
            'attribute',
            'questionset',
            'is_collection',
            'is_optional',
            'order',
            'default_option',
            'default_external_id',
            'widget_type',
            'value_type',
            'minimum',
            'maximum',
            'step',
            'unit',
            'width',
            'optionsets',
            'conditions'
        )
        trans_fields = (
            'help',
            'text',
            'default_text',
            'verbose_name',
            'verbose_name_plural',
        )

    def get_optionsets(self, obj):
        return [optionset.uri for optionset in obj.optionsets.all()]

    def get_conditions(self, obj):
        return [condition.uri for condition in obj.conditions.all()]


class QuestionSetExportSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    attribute = serializers.CharField(source='attribute.uri', default=None, read_only=True)
    section = serializers.CharField(source='section.uri', default=None, read_only=True)
    questionset = serializers.CharField(source='questionset.uri', default=None, read_only=True)
    questionsets = serializers.SerializerMethodField()
    questions = QuestionExportSerializer(many=True, read_only=True)
    conditions = serializers.SerializerMethodField()

    class Meta:
        model = QuestionSet
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'path',
            'comment',
            'attribute',
            'section',
            'questionset',
            'is_collection',
            'order',
            'questionsets',
            'questions',
            'conditions'
        )
        trans_fields = (
            'title',
            'help',
            'verbose_name',
            'verbose_name_plural',
        )

    def get_questionsets(self, obj):
        return QuestionSetExportSerializer(obj.questionsets, many=True).data

    def get_conditions(self, obj):
        return [condition.uri for condition in obj.conditions.all()]


class SectionExportSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    catalog = serializers.CharField(source='catalog.uri', default=None, read_only=True)
    questionsets = QuestionSetExportSerializer(many=True)

    class Meta:
        model = Section
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'path',
            'comment',
            'catalog',
            'order',
            'questionsets'
        )
        trans_fields = (
            'title',
        )


class CatalogExportSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    sections = SectionExportSerializer(many=True)

    class Meta:
        model = Catalog
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'order',
            'sections'
        )
        trans_fields = (
            'title',
            'help'
        )



# Generated by Django 2.2rc1 on 2019-03-26 13:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0042_remove_null_true'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='catalog',
            options={'ordering': ('order',), 'verbose_name': 'Catalog', 'verbose_name_plural': 'Catalogs'},
        ),
        migrations.AlterModelOptions(
            name='question',
            options={'ordering': ('questionset', 'order'), 'verbose_name': 'Question', 'verbose_name_plural': 'Questions'},
        ),
        migrations.AlterModelOptions(
            name='questionset',
            options={'ordering': ('section', 'order'), 'verbose_name': 'Question set', 'verbose_name_plural': 'Question set'},
        ),
        migrations.AlterModelOptions(
            name='section',
            options={'ordering': ('catalog__order', 'order'), 'verbose_name': 'Section', 'verbose_name_plural': 'Sections'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-03 14:08
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0004_full_title'),
    ]

    operations = [
        migrations.RenameField(
            model_name='section',
            old_name='full_title_de',
            new_name='label_de',
        ),
        migrations.RenameField(
            model_name='section',
            old_name='full_title_en',
            new_name='label_en',
        ),
        migrations.RenameField(
            model_name='subsection',
            old_name='full_title_de',
            new_name='label_de',
        ),
        migrations.RenameField(
            model_name='subsection',
            old_name='full_title_en',
            new_name='label_en',
        ),
        migrations.AddField(
            model_name='questionentity',
            name='label_de',
            field=models.CharField(default='', max_length=2048),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='questionentity',
            name='label_en',
            field=models.CharField(default='', max_length=2048),
            preserve_default=False,
        ),
    ]



from django.utils.translation import gettext_lazy as _

from rdmo.core.validators import (InstanceValidator, LockedValidator,
                                  UniqueURIValidator)

from .models import Attribute


class AttributeUniqueURIValidator(UniqueURIValidator):

    model = Attribute

    def get_uri(self, data):
        if not data.get('key'):
            self.raise_validation_error({'key': _('This field is required.')})
        else:
            path = self.model.build_path(data.get('key'), data.get('parent'))
            uri = self.model.build_uri(data.get('uri_prefix'), path)
            return uri


class AttributeParentValidator(InstanceValidator):

    def __call__(self, data):
        parent = data.get('parent')
        if parent:
            if self.serializer:
                # check copied attributes
                view = self.serializer.context.get('view')
                if view and view.action == 'copy':
                    # get the original from the view when cloning an attribute
                    if parent in view.get_object().get_descendants(include_self=True):
                        self.raise_validation_error({
                            'parent': [_('An attribute may not be cloned to be a child of itself or one of its descendants.')]
                        })

            # only check updated attributes
            if self.instance:
                if parent in self.instance.get_descendants(include_self=True):
                    self.raise_validation_error({
                        'parent': [_('An attribute may not be moved to be a child of itself or one of its descendants.')]
                    })


class AttributeLockedValidator(LockedValidator):

    parent_field = 'parent'



from rdmo.core.plugins import get_plugins, get_plugin


def get_widgets():
    widgets = get_plugins('QUESTIONS_WIDGETS')
    return widgets.values()


def get_widget_types():
    widgets = get_plugins('QUESTIONS_WIDGETS')
    return [widget.key for widget in widgets.values()]


def get_widget_type_choices():
    widgets = get_plugins('QUESTIONS_WIDGETS')
    return [(widget.key, widget.label) for widget in widgets.values()]


def get_widget_class(key):
    widget = get_plugin('QUESTIONS_WIDGETS', key)
    return widget.widget_class



from django.conf import settings

from .utils import import_class


class Plugin(object):

    def __init__(self, key, label, class_name):
        self.key = key
        self.label = label
        self.class_name = class_name


def get_plugins(plugin_settings):
    plugins = {}
    for key, label, class_name in getattr(settings, plugin_settings):
        plugins[key] = import_class(class_name)(key, label, class_name)
    return plugins


def get_plugin(plugin_settings, plugin_key):
    try:
        key, label, class_name = next(
            (key, label, class_name) for key, label, class_name in getattr(settings, plugin_settings) if key == plugin_key
        )
        return import_class(class_name)(key, label, class_name)
    except StopIteration:
        return None



# Generated by Django 2.2rc1 on 2019-03-26 13:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0019_remove_null_true'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='option',
            options={'ordering': ('optionset__order', 'optionset__key', 'order', 'key'), 'verbose_name': 'Option', 'verbose_name_plural': 'Options'},
        ),
        migrations.AlterModelOptions(
            name='optionset',
            options={'ordering': ('uri',), 'verbose_name': 'Option set', 'verbose_name_plural': 'Option sets'},
        ),
        migrations.AlterField(
            model_name='option',
            name='optionset',
            field=models.ForeignKey(blank=True, help_text='The option set this option belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='options', to='options.OptionSet', verbose_name='Option set'),
        ),
    ]



import os
import re
from pathlib import Path

import pytest
from django.urls import reverse

from rdmo.core.constants import VALUE_TYPE_FILE

from ..models import Project, Value

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

change_project_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

projects = [1, 2, 3, 4, 5]


@pytest.mark.parametrize('username,password', users)
def test_project_create_import_get(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('project_create_import')
    response = client.get(url)
    if password:
        assert response.status_code == 302
        assert response.url == '/projects/'
    else:
        assert response.status_code == 302
        assert response.url.startswith('/account/login/')


@pytest.mark.parametrize('username,password', users)
def test_project_create_import_post_error(db, settings, client, username, password):
    client.login(username=username, password=password)

    url = reverse('project_create_import')
    response = client.post(url, {
        'method': 'wrong'
    })

    if password:
        assert response.status_code == 400
    else:
        assert response.status_code == 302
        assert response.url.startswith('/account/login/')


@pytest.mark.parametrize('username,password', users)
def test_project_create_import_post_upload_file(db, settings, client, username, password):
    client.login(username=username, password=password)

    url = reverse('project_create_import')
    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'project.xml')
    with open(xml_file, encoding='utf8') as f:
        response = client.post(url, {
            'method': 'upload_file',
            'uploaded_file': f
        })

    if password:
        assert response.status_code == 200
        assert b'Create project from project.xml' in response.content
    else:
        assert response.status_code == 302
        assert response.url.startswith('/account/login/')


@pytest.mark.parametrize('username,password', users)
def test_project_create_import_post_upload_file_error(db, settings, client, username, password):
    client.login(username=username, password=password)

    url = reverse('project_create_import')
    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'error.xml')
    with open(xml_file, encoding='utf8') as f:
        response = client.post(url, {
            'method': 'upload_file',
            'uploaded_file': f
        })

    if password:
        assert response.status_code == 400
        assert b'Files of this type cannot be imported.' in response.content
    else:
        assert response.status_code == 302
        assert response.url.startswith('/account/login/')


@pytest.mark.parametrize('username,password', users)
def test_project_create_import_post_upload_file_empty(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('project_create_import')
    response = client.post(url, {
        'method': 'upload_file'
    })
    if password:
        assert response.status_code == 400
        assert b'There has been an error with your import.' in response.content
    else:
        assert response.status_code == 302
        assert response.url.startswith('/account/login/')


@pytest.mark.parametrize('username,password', users)
def test_project_create_import_post_import_file(db, settings, client, files, username, password):
    client.login(username=username, password=password)
    projects_count = Project.objects.count()

    # upload file
    url = reverse('project_create_import')
    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'project.xml')
    with open(xml_file, encoding='utf8') as f:
        response = client.post(url, {
            'method': 'upload_file',
            'uploaded_file': f
        })

    if password:
        assert response.status_code == 200

        # get keys from the response
        keys = re.findall(r'name=\"(.*?)\"', response.content.decode())

        # import file
        data = {key: ['on'] for key in keys}
        data.update({'method': 'import_file'})
        response = client.post(url, data)

        # check if all the files are where are supposed to be
        for file_value in Value.objects.filter(value_type=VALUE_TYPE_FILE):
            assert Path(settings.MEDIA_ROOT).joinpath(file_value.file.name).exists()

        # assert that the project exists and that there are values
        if password:
            project = Project.objects.order_by('updated').last()
            assert response.status_code == 302
            assert response.url == '/projects/{}/'.format(project.pk)

            # a new project, new values values
            assert Project.objects.count() == projects_count + 1
            assert project.values.count() > 0
        else:
            assert response.status_code == 302
            assert response.url.startswith('/account/login/')

            # no new project was created
            assert Project.objects.count() == projects_count
    else:
        assert response.status_code == 302
        assert response.url.startswith('/account/login/')


@pytest.mark.parametrize('username,password', users)
def test_project_create_import_post_empty(db, settings, client, username, password):
    client.login(username=username, password=password)
    projects_count = Project.objects.count()

    # upload file
    url = reverse('project_create_import')
    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'project.xml')
    with open(xml_file, encoding='utf8') as f:
        response = client.post(url, {
            'method': 'upload_file',
            'uploaded_file': f
        })

    if password:
        assert response.status_code == 200

        response = client.post(url, {
            'method': 'import_file'
        })

        # check if all the files are where are supposed to be
        for file_value in Value.objects.filter(value_type=VALUE_TYPE_FILE):
            assert Path(settings.MEDIA_ROOT).joinpath(file_value.file.name).exists()

        # assert that the project exists, but that there are not values
        if password:
            new_project = Project.objects.order_by('updated').last()
            assert response.status_code == 302
            assert response.url == '/projects/{}/'.format(new_project.id)

            # a new project, but no values
            assert Project.objects.count() == projects_count + 1
            assert new_project.values.count() == 0
        else:
            assert response.status_code == 302
            assert response.url.startswith('/account/login/')

            # no new project was created
            assert Project.objects.count() == projects_count
    else:
        assert response.status_code == 302
        assert response.url.startswith('/account/login/')


@pytest.mark.parametrize('username,password', users)
def test_project_create_import_post_import_project(db, settings, client, username, password):
    client.login(username=username, password=password)

    url = reverse('project_create_import')
    response = client.post(url, {
        'method': 'import_project'
    })

    if password:
        assert response.status_code == 400
    else:
        assert response.status_code == 302
        assert response.url.startswith('/account/login/')



from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet

from rdmo.core.exports import XMLResponse
from rdmo.core.permissions import HasModelPermission
from rdmo.core.viewsets import CopyModelMixin

from .models import View
from .renderers import ViewRenderer
from .serializers.export import ViewExportSerializer
from .serializers.v1 import ViewIndexSerializer, ViewSerializer


class ViewViewSet(CopyModelMixin, ModelViewSet):
    permission_classes = (HasModelPermission, )
    queryset = View.objects.prefetch_related('catalogs', 'sites', 'groups')
    serializer_class = ViewSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'uri',
        'key',
        'comment'
    )

    @action(detail=False)
    def index(self, request):
        serializer = ViewIndexSerializer(self.get_queryset(), many=True)
        return Response(serializer.data)

    @action(detail=False, permission_classes=[HasModelPermission])
    def export(self, request):
        serializer = ViewExportSerializer(self.get_queryset(), many=True)
        xml = ViewRenderer().render(serializer.data)
        return XMLResponse(xml, name='views')

    @action(detail=True, url_path='export', permission_classes=[HasModelPermission])
    def detail_export(self, request, pk=None):
        serializer = ViewExportSerializer(self.get_object())
        xml = ViewRenderer().render([serializer.data])
        return XMLResponse(xml, name=self.get_object().key)



import logging

from django.views.generic import DeleteView

from rdmo.core.views import ObjectPermissionMixin, RedirectViewMixin

from ..models import Invite

logger = logging.getLogger(__name__)


class InviteDeleteView(ObjectPermissionMixin, RedirectViewMixin, DeleteView):
    permission_required = 'projects.delete_invite_object'

    def get_queryset(self):
        return Invite.objects.filter(project_id=self.kwargs.get('project_id'))

    def get_permission_object(self):
        return self.get_object().project

    def get_success_url(self):
        return self.get_object().project.get_absolute_url()



from shutil import copyfile
from pathlib import Path

from django.apps import apps
from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def setup(self, options):
        self.theme_name = options['name']
        self.theme_path = Path(options['name'])
        self.rdmo_path = Path(apps.get_app_config('rdmo').path)
        self.local_path = Path().cwd() / 'config' / 'settings' / 'local.py'

    def copy(self, path):
        source_path = self.rdmo_path / path
        target_path = self.theme_path / Path(*path.parts[1:])

        if target_path.exists():
            print('Skip {} -> {}. Target file exists.'.format(source_path, target_path))
        else:
            print('Copy {} -> {}.'.format(source_path, target_path))

            target_path.parent.mkdir(parents=True, exist_ok=True)
            copyfile(source_path, target_path)

    def enable_theme(self):
        settings_line = 'INSTALLED_APPS = [\'{}\'] + INSTALLED_APPS'.format(self.theme_name)
        replaced = False

        local_settings = self.local_path.read_text().splitlines()
        for i, line in enumerate(local_settings):
            if line == settings_line:
                # return if the line is already there
                return

            if line == '# ' + settings_line:
                local_settings[i] = settings_line
                replaced = True

        if not replaced:
            local_settings.append('')
            local_settings.append(settings_line)
            local_settings.append('')

        self.local_path.write_text('\n'.join(local_settings))

    def add_arguments(self, parser):
        parser.add_argument('--name', action='store', default='rdmo_theme', help='Module name for the theme.')
        parser.add_argument('--file', action='store', help='Copy specific file/template, e.g. core/static/css/variables.scss.')

    def handle(self, *args, **options):
        self.setup(options)

        if options['file']:
            self.copy(Path(options['file']))
        else:
            self.theme_path.mkdir(exist_ok=True)
            self.theme_path.joinpath('__init__.py').touch()
            self.theme_path.joinpath('locale').mkdir(exist_ok=True)

            self.copy(Path('core') / 'static' / 'core' / 'css' / 'variables.scss')

            for language, language_string in settings.LANGUAGES:
                self.copy(Path('core') / 'templates' / 'core' / 'home_text_{}.html'.format(language))
                self.copy(Path('core') / 'templates' / 'core' / 'about_text_{}.html'.format(language))
                self.copy(Path('core') / 'templates' / 'core' / 'footer_text_{}.html'.format(language))

            print('Enable theme by adding the necessary config line.')
            self.enable_theme()

        print('Done')



from django.urls import include, path

from rest_framework import routers

from ..viewsets import ConditionViewSet, RelationViewSet

app_name = 'v1-conditions'

router = routers.DefaultRouter()
router.register(r'conditions', ConditionViewSet, basename='condition')
router.register(r'relations', RelationViewSet, basename='relation')

urlpatterns = [
    path('', include(router.urls)),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-21 14:16
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0013_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='project',
            name='owner',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-16 14:01
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0013_meta'),
    ]

    operations = [
        migrations.AlterField(
            model_name='condition',
            name='source',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='The Attribute of the value this condition.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute', verbose_name='Source'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='target_option',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='If using a value pointing to an option, the option this condition is checking against.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='options.Option', verbose_name='Target (Option)'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='target_text',
            field=models.CharField(blank=True, help_text='If using a regular value, the text value this condition is checking against (for boolean values use 1 and 0).', max_length=256, null=True, verbose_name='Target (Text)'),
        ),
    ]



import logging

from rdmo.core.imports import (fetch_parents, get_foreign_field,
                               set_common_fields, validate_instance)

from .models import Attribute
from .validators import (AttributeLockedValidator, AttributeParentValidator,
                         AttributeUniqueURIValidator)

logger = logging.getLogger(__name__)


def import_attribute(element, parent_uri=False, save=False):
    try:
        if parent_uri is False:
            attribute = Attribute.objects.get(uri=element.get('uri'))
        else:
            attribute = Attribute.objects.get(key=element.get('key'), parent__uri=parent_uri)
    except Attribute.DoesNotExist:
        attribute = Attribute()

    set_common_fields(attribute, element)

    attribute.parent = get_foreign_field(attribute, parent_uri or element.get('parent'), Attribute)

    if save and validate_instance(attribute,
                                  AttributeLockedValidator,
                                  AttributeParentValidator,
                                  AttributeUniqueURIValidator):
        if attribute.id:
            logger.debug('Attribute created with uri %s.', element.get('uri'))
        else:
            logger.debug('Attribute %s updated.', element.get('uri'))

        attribute.save()
        attribute.imported = True

    return attribute


def fetch_attribute_parents(instances):
    return fetch_parents(Attribute, instances)



from django.urls import re_path

from ..views import TasksExportView, TasksView

urlpatterns = [
    re_path(r'^$', TasksView.as_view(), name='tasks'),
    re_path(r'^export/(?P<format>[a-z]+)/$', TasksExportView.as_view(), name='tasks_export'),
]



# Generated by Django 2.2.13 on 2020-10-02 16:35

from django.db import migrations
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0045_require_uri_prefix'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, help_text='Parent attribute in the domain model.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='domain.Attribute', verbose_name='Parent attribute'),
        ),
    ]



from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from rdmo.conditions.models import Condition
from rdmo.core.serializers import MarkdownSerializerMixin
from rdmo.options.models import Option, OptionSet
from rdmo.questions.models import Question, QuestionSet
from rdmo.questions.utils import get_widget_class


class OptionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Option
        fields = (
            'id',
            'optionset',
            'text',
            'additional_input'
        )


class OptionSetSerializer(serializers.ModelSerializer):

    options = OptionSerializer(many=True)

    class Meta:
        model = OptionSet
        fields = (
            'id',
            'options',
            'has_provider',
            'has_search',
            'has_conditions'
        )


class ConditionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Condition
        fields = (
            'id',
            'source',
            'relation',
            'target_text',
            'target_option'
        )


class QuestionSerializer(MarkdownSerializerMixin, serializers.ModelSerializer):

    markdown_fields = ('help', )

    conditions = ConditionSerializer(default=None, many=True)
    optionsets = serializers.SerializerMethodField()

    verbose_name = serializers.SerializerMethodField()
    verbose_name_plural = serializers.SerializerMethodField()
    widget_class = serializers.SerializerMethodField()

    class Meta:
        model = Question
        fields = (
            'id',
            'order',
            'help',
            'text',
            'default_text',
            'default_option',
            'default_external_id',
            'verbose_name',
            'verbose_name_plural',
            'widget_type',
            'widget_class',
            'value_type',
            'unit',
            'width',
            'minimum',
            'maximum',
            'step',
            'attribute',
            'conditions',
            'optionsets',
            'is_collection',
            'is_optional',
            'has_conditions'
        )

    def get_optionsets(self, obj):
        return OptionSetSerializer(obj.optionsets.order_by('order'), many=True).data

    def get_verbose_name(self, obj):
        return obj.verbose_name or _('item')

    def get_verbose_name_plural(self, obj):
        return obj.verbose_name_plural or _('items')

    def get_widget_class(self, obj):
        return get_widget_class(obj.widget_type)


class QuestionSetSerializer(MarkdownSerializerMixin, serializers.ModelSerializer):

    markdown_fields = ('help', )

    questionsets = serializers.SerializerMethodField()
    questions = QuestionSerializer(many=True)

    section = serializers.SerializerMethodField()

    verbose_name = serializers.SerializerMethodField()
    verbose_name_plural = serializers.SerializerMethodField()

    class Meta:
        model = QuestionSet
        fields = (
            'id',
            'order',
            'title',
            'help',
            'verbose_name',
            'verbose_name_plural',
            'attribute',
            'is_collection',
            'next',
            'prev',
            'section',
            'questionsets',
            'questions',
            'has_conditions'
        )

    def get_questionsets(self, obj):
        return QuestionSetSerializer(obj.questionsets.all(), many=True, read_only=True).data

    def get_section(self, obj):
        return {
            'id': obj.section.id,
            'title': obj.section.title
        }

    def get_verbose_name(self, obj):
        return obj.verbose_name or _('set')

    def get_verbose_name_plural(self, obj):
        return obj.verbose_name_plural or _('sets')



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-25 11:32
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0021_options'),
        ('options', '0003_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='option',
            name='attribute',
        ),
        migrations.DeleteModel(
            name='Option',
        ),
    ]



# Generated by Django 2.2.13 on 2020-08-31 16:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0031_related_name'),
        ('projects', '0036_remove_project_tasks'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='tasks',
            field=models.ManyToManyField(blank=True, help_text='The tasks that will be used for this project.', through='projects.Issue', to='tasks.Task', verbose_name='Tasks'),
        ),
    ]



import logging

from django.db import models
from django.utils.timezone import now
from django.utils.translation import get_language
from django.utils.translation import gettext_lazy as _

from rdmo.core.utils import get_languages

logger = logging.getLogger(__name__)


class Model(models.Model):

    created = models.DateTimeField(editable=False, verbose_name=_('created'))
    updated = models.DateTimeField(editable=False, verbose_name=_('updated'))

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if self.created is None:
            self.created = now()

        self.updated = now()

        super(Model, self).save(*args, **kwargs)


class TranslationMixin(object):

    def trans(self, field):
        current_language = get_language()

        languages = get_languages()
        for lang_code, lang_string, lang_field in languages:
            if lang_code == current_language:
                r = getattr(self, '%s_%s' % (field, lang_field)) or None
                if r is not None:
                    return r
                else:
                    for i in range(1, 6):
                        r = getattr(self, '%s_%s' % (field, 'lang' + str(i))) or None
                        if r is not None:
                            return r
        return ''



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Catalog
from ..serializers.v1 import CatalogSerializer
from ..validators import CatalogUniqueURIValidator


def test_unique_uri_validator_create(db):
    CatalogUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_unique_uri_validator_create_error(db):
    with pytest.raises(ValidationError):
        CatalogUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': Catalog.objects.get(pk=2).key
        })


def test_unique_uri_validator_update(db):
    catalog = Catalog.objects.get(pk=1)

    CatalogUniqueURIValidator(catalog)({
        'uri_prefix': catalog.uri_prefix,
        'key': catalog.key
    })


def test_unique_uri_validator_update_error(db):
    catalog = Catalog.objects.get(pk=1)

    with pytest.raises(ValidationError):
        CatalogUniqueURIValidator(catalog)({
            'uri_prefix': catalog.uri_prefix,
            'key': Catalog.objects.get(pk=2).key
        })


def test_unique_uri_validator_serializer_create(db):
    validator = CatalogUniqueURIValidator()
    validator.set_context(CatalogSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_unique_uri_validator_serializer_create_error(db):
    validator = CatalogUniqueURIValidator()
    validator.set_context(CatalogSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': Catalog.objects.get(pk=2).key
        })


def test_unique_uri_validator_serializer_update(db):
    catalog = Catalog.objects.get(pk=1)

    validator = CatalogUniqueURIValidator()
    validator.set_context(CatalogSerializer(instance=catalog))

    validator({
        'uri_prefix': catalog.uri_prefix,
        'key': catalog.key
    })


def test_unique_uri_validator_serializer_update_error(db):
    catalog = Catalog.objects.get(pk=1)

    validator = CatalogUniqueURIValidator()
    validator.set_context(CatalogSerializer(instance=catalog))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': catalog.uri_prefix,
            'key': Catalog.objects.get(pk=2).key
        })



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-06-06 15:09
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0031_meta'),
        ('projects', '0018_data_migration'),
        ('questions', '0016_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='attribute',
            name='unit',
        ),
        migrations.RemoveField(
            model_name='attribute',
            name='value_type',
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations


def run_data_migration(apps, schema_editor):
    Attribute = apps.get_model('domain', 'Attribute')
    QuestionSet = apps.get_model('questions', 'QuestionSet')

    questionsets = QuestionSet.objects.filter(is_collection=True)
    for questionset in questionsets:
        if questionset.attribute.key != 'id':
            try:
                questionset.attribute = Attribute.objects.get(parent=questionset.attribute, key='id')
                questionset.save()
            except Attribute.DoesNotExist:
                pass


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0064_widget_type_choices'),
    ]

    operations = [
        migrations.RunPython(run_data_migration),
    ]



import pytest
from django.contrib.auth import get_user_model
from django.urls import reverse

from ..models import Membership

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('api', 'api'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

view_membership_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'author': [1, 3, 5],
    'guest': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

add_membership_permission_map = change_membership_permission_map = delete_membership_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

urlnames = {
    'list': 'v1-projects:project-membership-list',
    'detail': 'v1-projects:project-membership-detail'
}

projects = [1, 2, 3, 4, 5]
memberships = [1, 2, 3, 4]
membership_roles = ('owner', 'manager', 'author', 'guest')


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_list(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'], args=[project_id])
    response = client.get(url)

    if project_id in view_membership_permission_map.get(username, []):
        assert response.status_code == 200

        if username == 'user':
            assert sorted([item['id'] for item in response.json()]) == []
        else:
            values_list = Membership.objects.filter(project_id=project_id) \
                                            .order_by('id').values_list('id', flat=True)
            assert sorted([item['id'] for item in response.json()]) == list(values_list)
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_id', memberships)
def test_detail(db, client, username, password, project_id, membership_id):
    client.login(username=username, password=password)
    membership = Membership.objects.filter(project_id=project_id, id=membership_id).first()

    url = reverse(urlnames['detail'], args=[project_id, membership_id])
    response = client.get(url)

    if membership and project_id in view_membership_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)
        assert response.json().get('id') == membership_id
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_role', membership_roles)
def test_create(db, client, username, password, project_id, membership_role):
    client.login(username=username, password=password)

    user = get_user_model().objects.get(username='user')

    url = reverse(urlnames['list'], args=[project_id])
    data = {
        'user': user.pk,
        'role': membership_role
    }
    response = client.post(url, data)

    if project_id in add_membership_permission_map.get(username, []):
        assert response.status_code == 201
    elif project_id in view_membership_permission_map.get(username, []):
        assert response.status_code == 403
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_id', memberships)
@pytest.mark.parametrize('membership_role', membership_roles)
def test_update(db, client, username, password, project_id, membership_id, membership_role):
    client.login(username=username, password=password)
    membership = Membership.objects.filter(project_id=project_id, id=membership_id).first()

    url = reverse(urlnames['detail'], args=[project_id, membership_id])
    data = {
        'role': membership_role
    }
    response = client.put(url, data, content_type='application/json')

    if membership and project_id in change_membership_permission_map.get(username, []):
        assert response.status_code == 200
    elif membership and project_id in view_membership_permission_map.get(username, []):
        assert response.status_code == 403
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_id', memberships)
def test_delete(db, client, username, password, project_id, membership_id):
    client.login(username=username, password=password)
    membership = Membership.objects.filter(project_id=project_id, id=membership_id).first()

    url = reverse(urlnames['detail'], args=[project_id, membership_id])
    response = client.delete(url)

    if membership and project_id in delete_membership_permission_map.get(username, []):
        assert response.status_code == 204
    elif membership and project_id in view_membership_permission_map.get(username, []):
        assert response.status_code == 403
    else:
        assert response.status_code == 404



from pathlib import Path

import pytest
from django.conf import settings
from django.urls import reverse

from rdmo.core.constants import (VALUE_TYPE_CHOICES, VALUE_TYPE_FILE,
                                 VALUE_TYPE_TEXT)
from rdmo.questions.models import Question

from ..models import Value

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('api', 'api'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

view_value_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'author': [1, 3, 5],
    'guest': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

add_value_permission_map = change_value_permission_map = delete_value_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'author': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

urlnames = {
    'list': 'v1-projects:project-value-list',
    'detail': 'v1-projects:project-value-detail',
    'set': 'v1-projects:project-value-set',
    'file': 'v1-projects:project-value-file'
}

projects = [1, 2, 3, 4, 5]
values = [1, 2, 3, 4, 5, 6, 7, 238, 242, 247, 248, 249]

attribute_id = 1
option_id = 1

set_values = [84, 85]
set_questionsets = [42, 43]


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_list(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'], args=[project_id])
    response = client.get(url)

    if project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), list)

        if username == 'user':
            assert sorted([item['id'] for item in response.json()]) == []
        else:
            values_list = Value.objects.filter(project_id=project_id) \
                                       .filter(snapshot_id=None) \
                                       .order_by('id').values_list('id', flat=True)
            assert sorted([item['id'] for item in response.json()]) == list(values_list)

    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('value_id', values)
def test_detail(db, client, username, password, project_id, value_id):
    client.login(username=username, password=password)
    value = Value.objects.filter(project_id=project_id, id=value_id).filter()

    url = reverse(urlnames['detail'], args=[project_id, value_id])
    response = client.get(url)

    if value and project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)
        assert response.json().get('id') == value_id
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('value_type,value_type_label', VALUE_TYPE_CHOICES)
def test_create_text(db, client, username, password, project_id, value_type, value_type_label):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'], args=[project_id])
    data = {
        'attribute': attribute_id,
        'set_index': 0,
        'collection_index': 0,
        'text': 'Lorem ipsum',
        'value_type': value_type,
        'unit': ''
    }
    response = client.post(url, data)

    if project_id in add_value_permission_map.get(username, []):
        assert response.status_code == 201
        assert isinstance(response.json(), dict)
        assert response.json().get('id') in Value.objects.filter(project_id=project_id).values_list('id', flat=True)
    elif project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 403
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('value_type,value_type_label', VALUE_TYPE_CHOICES)
def test_create_option(db, client, username, password, project_id, value_type, value_type_label):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'], args=[project_id])
    data = {
        'attribute': attribute_id,
        'set_index': 0,
        'collection_index': 0,
        'option': option_id,
        'value_type': value_type,
        'unit': ''
    }
    response = client.post(url, data)

    if project_id in add_value_permission_map.get(username, []):
        assert response.status_code == 201
        assert isinstance(response.json(), dict)
        assert response.json().get('id') in Value.objects.filter(project_id=project_id).values_list('id', flat=True)
    elif project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 403
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('value_type,value_type_label', VALUE_TYPE_CHOICES)
def test_create_external(db, client, username, password, project_id, value_type, value_type_label):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'], args=[project_id])
    data = {
        'attribute': attribute_id,
        'set_index': 0,
        'collection_index': 0,
        'text': 'Lorem ipsum',
        'external_id': '1',
        'value_type': value_type,
        'unit': ''
    }
    response = client.post(url, data)

    if project_id in add_value_permission_map.get(username, []):
        assert response.status_code == 201
        assert isinstance(response.json(), dict)
        assert response.json().get('id') in Value.objects.filter(project_id=project_id).values_list('id', flat=True)
    elif project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 403
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('value_id', values)
def test_update(db, client, username, password, project_id, value_id):
    client.login(username=username, password=password)
    value = Value.objects.filter(project_id=project_id, id=value_id).first()

    url = reverse(urlnames['detail'], args=[project_id, value_id])
    data = {
        'attribute': attribute_id,
        'set_index': 0,
        'collection_index': 0,
        'text': 'Lorem ipsum',
        'value_type': VALUE_TYPE_TEXT,
        'unit': ''
    }
    response = client.put(url, data, content_type='application/json')

    if value and project_id in change_value_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)
        assert response.json().get('id') in Value.objects.filter(project_id=project_id).values_list('id', flat=True)
    elif value and project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 403
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('value_id', values)
def test_delete(db, client, username, password, project_id, value_id):
    client.login(username=username, password=password)
    value = Value.objects.filter(project_id=project_id, id=value_id).first()

    url = reverse(urlnames['detail'], args=[project_id, value_id])
    response = client.delete(url)

    if value and project_id in delete_value_permission_map.get(username, []):
        assert response.status_code == 204
        assert not Value.objects.filter(pk=value_id).exists()
    elif value and project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 403
        assert Value.objects.filter(pk=value_id).exists()
    else:
        assert response.status_code == 404
        assert Value.objects.filter(pk=value_id).exists()


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('value_id', set_values)
def test_set(db, client, username, password, project_id, value_id):
    client.login(username=username, password=password)
    value = Value.objects.filter(project_id=project_id, snapshot=None, id=value_id).first()

    set_attributes = Question.objects.filter(questionset__id__in=set_questionsets).values_list('attribute', flat=True)
    values_count = Value.objects.count()
    if value and project_id in delete_value_permission_map.get(username, []):
        set_values_count = Value.objects.filter(project_id=project_id,
                                                snapshot=None,
                                                attribute__in=set_attributes,
                                                set_index=value.set_index).count()

    url = reverse(urlnames['set'], args=[project_id, value_id])
    response = client.delete(url)

    if value and project_id in delete_value_permission_map.get(username, []):
        assert response.status_code == 204
        assert not Value.objects.filter(pk=value_id).exists()
        assert Value.objects.count() == values_count - set_values_count - 1  # one is for set/id
    elif value and project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 403
        assert Value.objects.filter(pk=value_id).exists()
        assert Value.objects.count() == values_count
    else:
        assert response.status_code == 404
        assert Value.objects.filter(pk=value_id).exists()
        assert Value.objects.count() == values_count


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('value_id', values)
def test_file_get(db, client, files, username, password, project_id, value_id):
    client.login(username=username, password=password)
    value = Value.objects.filter(project_id=project_id, id=value_id).first()

    url = reverse(urlnames['file'], args=[project_id, value_id])
    response = client.get(url)

    if value and value.value_type == VALUE_TYPE_FILE and project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 200
        assert response['Content-Type'] == value.file_type
        assert response['Content-Disposition'] == 'attachment; filename={}'.format(value.file_name)
        assert response.content == value.file.read()
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('value_id', values)
def test_file_put(db, client, files, username, password, project_id, value_id):
    client.login(username=username, password=password)
    value = Value.objects.filter(project_id=project_id, id=value_id).first()

    url = reverse(urlnames['file'], args=[project_id, value_id])

    file_path = Path(settings.MEDIA_ROOT) / 'test_file.txt'
    with file_path.open() as fp:
        response = client.post(url, {'name': 'test_file.txt', 'file': fp})

    if value and project_id in change_value_permission_map.get(username, []):
        assert response.status_code == 200
        assert response.json().get('file_name') == 'test_file.txt'
    elif value and project_id in view_value_permission_map.get(username, []):
        assert response.status_code == 403
    else:
        assert response.status_code == 404



from django.urls import include, path

from rest_framework import routers

from ..viewsets import ViewViewSet

app_name = 'v1-views'

router = routers.DefaultRouter()
router.register(r'views', ViewViewSet, basename='view')

urlpatterns = [
    path('', include(router.urls)),
]



# Generated by Django 2.2.6 on 2019-10-11 12:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0017_role'),
    ]

    operations = [
        migrations.AlterField(
            model_name='role',
            name='manager',
            field=models.ManyToManyField(blank=True, help_text='The sites for which this user is manager.', related_name='managers', to='sites.Site', verbose_name='Manager'),
        ),
        migrations.AlterField(
            model_name='role',
            name='member',
            field=models.ManyToManyField(blank=True, help_text='The sites for which this user is a member.', related_name='members', to='sites.Site', verbose_name='Member'),
        ),
    ]



# Generated by Django 2.2.16 on 2021-01-22 17:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0054_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='catalog',
            name='locked',
            field=models.BooleanField(default=False, help_text="Designates whether this catalog (and it's sections, question sets and questions) can be changed.", verbose_name='Locked'),
        ),
        migrations.AddField(
            model_name='question',
            name='locked',
            field=models.BooleanField(default=False, help_text='Designates whether this question can be changed.', verbose_name='Locked'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='locked',
            field=models.BooleanField(default=False, help_text="Designates whether this questionset (and it's questions) can be changed.", verbose_name='Locked'),
        ),
        migrations.AddField(
            model_name='section',
            name='locked',
            field=models.BooleanField(default=False, help_text="Designates whether this section (and it's question sets and questions) can be changed.", verbose_name='Locked'),
        ),
    ]



from django.contrib import admin

from .models import (Continuation, Integration, IntegrationOption, Invite,
                     Issue, IssueResource, Membership, Project, Snapshot,
                     Value)


class ProjectAdmin(admin.ModelAdmin):
    search_fields = ('title', 'user__username')
    list_display = ('title', 'owners')

    def owners(self, obj):
        return ', '.join([user.username for user in obj.owners])


class MembershipAdmin(admin.ModelAdmin):
    search_fields = ('project__title', 'user__username', 'role')
    list_display = ('project', 'user', 'role')


class ContinuationAdmin(admin.ModelAdmin):
    search_fields = ('project__title', 'user__username')
    list_display = ('project', 'user', 'questionset')


class IntegrationAdmin(admin.ModelAdmin):
    search_fields = ('project__title', 'provider_key')
    list_display = ('project', 'provider_key')


class IntegrationOptionAdmin(admin.ModelAdmin):
    search_fields = ('integration__project__title', 'key', 'value')
    list_display = ('integration', 'key', 'value')


class InviteAdmin(admin.ModelAdmin):
    search_fields = ('project__title', 'user__username', 'email', 'role')
    list_display = ('project', 'user', 'email', 'token', 'timestamp')
    readonly_fields = ('token', 'timestamp')


class IssueAdmin(admin.ModelAdmin):
    search_fields = ('project__title', 'task', 'status')
    list_display = ('project', 'task', 'status')
    list_filter = ('status', )


class IssueResourceAdmin(admin.ModelAdmin):
    search_fields = ('issue__project__title', 'url')
    list_display = ('issue', 'url')


class SnapshotAdmin(admin.ModelAdmin):
    search_fields = ('title', 'project__title', 'project__user__username')
    list_display = ('title', 'project')


class ValueAdmin(admin.ModelAdmin):
    search_fields = ('attribute__uri', 'project__title', 'snapshot__title', 'project__user__username')
    list_display = ('attribute', 'set_prefix', 'set_index', 'collection_index', 'project', 'snapshot_title')
    list_filter = ('value_type', )

    def snapshot_title(self, obj):
        if obj.snapshot:
            return obj.snapshot.title


admin.site.register(Project, ProjectAdmin)
admin.site.register(Membership, MembershipAdmin)
admin.site.register(Continuation, ContinuationAdmin)
admin.site.register(Integration, IntegrationAdmin)
admin.site.register(IntegrationOption, IntegrationOptionAdmin)
admin.site.register(Invite, InviteAdmin)
admin.site.register(Issue, IssueAdmin)
admin.site.register(IssueResource, IssueResourceAdmin)
admin.site.register(Snapshot, SnapshotAdmin)
admin.site.register(Value, ValueAdmin)



import hmac
import json

import pytest
from django.urls import reverse

from ..models import Integration, Issue

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

view_integration_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'author': [1, 3, 5],
    'guest': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

add_integration_permission_map = change_integration_permission_map = delete_integration_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

projects = [1, 2, 3, 4, 5]
integrations = [1, 2]


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_integration_create_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('integration_create', args=[project_id, 'github'])
    response = client.get(url)

    if project_id in add_integration_permission_map.get(username, []):
        assert response.status_code == 200
    elif password:
        assert response.status_code == 403
    else:
        assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_integration_create_post(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('integration_create', args=[project_id, 'github'])
    data = {
        'repo': 'example/example1'
    }
    response = client.post(url, data)

    if project_id in add_integration_permission_map.get(username, []):
        assert response.status_code == 302
        values = Integration.objects.order_by('id').last().options.values('key', 'value', 'secret')
        assert sorted(values, key=lambda obj: obj['key']) == [
            {
                'key': 'repo',
                'value': 'example/example1',
                'secret': False
            },
            {
                'key': 'secret',
                'value': '',
                'secret': True
            }
        ]
    elif password:
        assert response.status_code == 403
    else:
        assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('integration_id', integrations)
def test_integration_update_get(db, client, username, password, project_id, integration_id):
    client.login(username=username, password=password)
    integration = Integration.objects.filter(project_id=project_id, id=integration_id).first()

    url = reverse('integration_update', args=[project_id, integration_id])
    response = client.get(url)

    if integration:
        if project_id in change_integration_permission_map.get(username, []):
            assert response.status_code == 200
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('integration_id', integrations)
def test_integration_update_post(db, client, username, password, project_id, integration_id):
    client.login(username=username, password=password)
    integration = Integration.objects.filter(project_id=project_id, id=integration_id).first()

    url = reverse('integration_update', args=[project_id, integration_id])
    data = {
        'repo': 'example/example2',
        'secret': 'super_secret'
    }
    response = client.post(url, data)

    if integration:
        if project_id in change_integration_permission_map.get(username, []):
            assert response.status_code == 302
            values = Integration.objects.filter(project_id=project_id, id=integration_id).first().options.values('key', 'value', 'secret')
            assert sorted(values, key=lambda obj: obj['key']) == [
                {
                    'key': 'repo',
                    'value': 'example/example2',
                    'secret': False
                },
                {
                    'key': 'secret',
                    'value': 'super_secret',
                    'secret': True
                }
            ]
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('integration_id', integrations)
def test_integration_delete_get(db, client, username, password, project_id, integration_id):
    client.login(username=username, password=password)
    integration = Integration.objects.filter(project_id=project_id, id=integration_id).first()

    url = reverse('integration_delete', args=[project_id, integration_id])
    response = client.get(url)

    if integration:
        if project_id in delete_integration_permission_map.get(username, []):
            assert response.status_code == 200
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('integration_id', integrations)
def test_integration_delete_post(db, client, username, password, project_id, integration_id):
    client.login(username=username, password=password)
    integration = Integration.objects.filter(project_id=project_id, id=integration_id).first()

    url = reverse('integration_delete', args=[project_id, integration_id])
    response = client.delete(url)

    if integration:
        if project_id in delete_integration_permission_map.get(username, []):
            assert response.status_code == 302
            assert Integration.objects.filter(project_id=project_id, id=integration_id).first() is None
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('integration_id', integrations)
def test_integration_webhook_get(db, client, project_id, integration_id):
    url = reverse('integration_webhook', args=[project_id, integration_id])
    response = client.get(url)

    assert response.status_code == 405
    assert Issue.objects.filter(status='closed').count() == 0


@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('integration_id', integrations)
def test_integration_webhook_post(db, client, project_id, integration_id):
    integration = Integration.objects.filter(project_id=project_id, id=integration_id).first()

    secret = 'super_duper_secret'
    url = reverse('integration_webhook', args=[project_id, integration_id])
    data = {
        'action': 'closed',
        'issue': {
            'html_url': 'https://github.com/example/example/issues/1'
        }
    }
    body = json.dumps(data)
    signature = 'sha1=' + hmac.new(secret.encode(), body.encode(), 'sha1').hexdigest()

    response = client.post(url, data, **{'HTTP_X_HUB_SIGNATURE': signature, 'content_type': 'application/json'})

    if integration:
        assert response.status_code == 200
        assert Issue.objects.filter(status='closed').count() == (1 if integration_id == 1 else 0)
    else:
        assert response.status_code == 404
        assert Issue.objects.filter(status='closed').count() == 0


@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('integration_id', integrations)
def test_integration_webhook_post_no_secret(db, client, project_id, integration_id):
    url = reverse('integration_webhook', args=[project_id, integration_id])
    response = client.post(url, {})

    assert response.status_code == 404
    assert Issue.objects.filter(status='closed').count() == 0



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-09-28 11:27
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0007_data_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='value',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='values', to='projects.Project'),
        ),
    ]



from collections import defaultdict

from django.utils.functional import cached_property
from mptt.utils import get_cached_trees

from rdmo.conditions.models import Condition


class ProjectWrapper(object):

    def __init__(self, project, snapshot=None):
        self._project = project
        self._catalog = project.catalog
        self._snapshot = snapshot
        self._checked_conditions = defaultdict(lambda: defaultdict(lambda: defaultdict(list)))

    def __str__(self):
        return str(self._project.title)

    @property
    def id(self):
        return self._project.id

    @property
    def title(self):
        return self._project.title

    @property
    def description(self):
        return self._project.description

    @property
    def created(self):
        return self._project.created

    @property
    def updated(self):
        return self._project.updated

    @property
    def snapshot(self):
        if self._snapshot:
            return {
                'id': self._snapshot.id,
                'title': self._snapshot.title,
                'description': self._snapshot.description,
                'created': self._snapshot.created,
                'updated': self._snapshot.updated
            }
        else:
            return {}

    @cached_property
    def descendants(self):
        return [ProjectWrapper(descendant) for descendant in self._project.get_descendants()]

    @cached_property
    def children(self):
        return [ProjectWrapper(child) for child in self._project.get_children()]

    @cached_property
    def tree(self):
        cached_trees = get_cached_trees(self._project.get_descendants())
        return self._build_tree(cached_trees)

    @cached_property
    def conditions(self):
        conditions = {}
        for condition in self._conditions:
            conditions[condition.uri] = conditions[condition.key] = self._check_condition(condition)
        return conditions

    @cached_property
    def _values(self):
        return list(self._project.values.filter(snapshot=self._snapshot).select_related('attribute', 'option'))

    @cached_property
    def _conditions(self):
        return list(Condition.objects.select_related('source', 'target_option').prefetch_related(
            'questions',
            'questionsets'
        ))

    def _check_conditions(self, conditions, set_prefix=None, set_index=None):
        # caches the result of the check in the wrapper
        if conditions:
            for condition in conditions:
                if self._check_condition(condition, set_prefix, set_index):
                    return True
            return False
        else:
            return True

    def _check_condition(self, condition, set_prefix=None, set_index=None):
        # caches the result of the check in the wrapper
        if self._checked_conditions[condition.id][set_prefix][set_index] == []:
            # find the question in the catalog, to resolve complex value structures
            from rdmo.questions.models import Question
            question = Question.objects.filter_by_catalog(self._catalog).filter(attribute=condition.source).first()

            self._checked_conditions[condition.id][set_prefix][set_index] = \
                condition.resolve(self._values, set_prefix, set_index, question)

        return self._checked_conditions[condition.id][set_prefix][set_index]

    def _build_tree(self, projects):
        return [{
            'id': project.id,
            'level': project.level,
            'children': self.build_tree(project.get_children())
        } for project in projects]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-07-22 11:37
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('domain', '0008_meta'),
    ]

    operations = [
        migrations.CreateModel(
            name='Condition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('relation', models.CharField(choices=[('eq', 'equal (==)'), ('neq', 'not equal (!=)'), ('contains', 'contains'), ('gt', 'greater than (>)'), ('gte', 'greater than or equal (>=)'), ('lt', 'lesser than (<)'), ('lte', 'lesser than or equal (<=)')], max_length=8)),
                ('target_text', models.CharField(blank=True, max_length=256, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='contenttypes.ContentType')),
                ('source', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute')),
                ('target_option', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Option')),
            ],
            options={
                'verbose_name': 'Condition',
                'verbose_name_plural': 'Conditions',
            },
        ),
    ]



from django.core.management.base import BaseCommand, CommandError

from rdmo.projects.models import Project, Membership


class Command(BaseCommand):
    help = 'Search and remove projects without users with specific role'


    def add_arguments(self, parser):
        parser.add_argument('--min_role', type=str, default='owner', \
            help='Minimum membership role for projects to be pruned, e.g. author ' + \
                'will remove projects without author, manager and owner. (Default: owner)')
        parser.add_argument('--remove', action='store_true', \
            help='Set this flag to actually remove projects')


    def handle(self, *args, **options):
        roles = ['owner']

        if options['min_role'] == 'guest':
            roles.extend(['guest', 'author', 'manager'])
        elif options['min_role'] == 'author':
            roles.extend(['author', 'manager'])
        elif options['min_role'] == 'manager':
            roles.extend(['manager'])
        elif options['min_role'] != 'owner':
            raise CommandError('Role "%s" does not exist' % options['min_role'])

        memberships = Membership.objects.filter(role__in=roles).values_list('pk')
        candidates = Project.objects.exclude(memberships__in=list(memberships)).distinct()

        if candidates.count() == 0:
            self.stdout.write(self.style.SUCCESS('No projects without %s' % (roles)))
            return

        self.stdout.write('Found projects without %s:' % (roles))
        for proj in candidates:
            self.stdout.write('%s (id=%s)' % (proj, proj.id))
            if options['remove']:
                self.stdout.write('...removing...', ending='')
                proj.delete()
                self.stdout.write(self.style.SUCCESS('OK'))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-31 12:40
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0020_data_migration'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='value',
            options={'ordering': ('attribute', 'set_index', 'collection_index'), 'permissions': (('view_value', 'Can view value'),), 'verbose_name': 'Value', 'verbose_name_plural': 'Values'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-03 15:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import rdmo.core.models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0035_remove_is_collection_and_parent_collection'),
        ('options', '0013_order'),
        ('conditions', '0014_meta'),
        ('questions', '0024_data_migration'),
    ]

    operations = [
        migrations.CreateModel(
            name='QuestionItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(editable=False, verbose_name='created')),
                ('updated', models.DateTimeField(editable=False, verbose_name='updated')),
                ('uri', models.URLField(blank=True, help_text='The Uniform Resource Identifier of this question (auto-generated).', max_length=640, null=True, verbose_name='URI')),
                ('uri_prefix', models.URLField(blank=True, help_text='The prefix for the URI of this question.', max_length=256, null=True, verbose_name='URI Prefix')),
                ('key', models.SlugField(blank=True, help_text='The internal identifier of this question.', max_length=128, null=True, verbose_name='Key')),
                ('path', models.CharField(blank=True, help_text='The path part of the URI of this question (auto-generated).', max_length=512, null=True, verbose_name='Path')),
                ('comment', models.TextField(blank=True, help_text='Additional internal information about this question.', null=True, verbose_name='Comment')),
                ('is_collection', models.BooleanField(default=False, help_text='Designates whether this question is a collection.', verbose_name='is collection')),
                ('order', models.IntegerField(default=0, help_text='The position of this question in lists.', verbose_name='Order')),
                ('help_en', models.TextField(blank=True, help_text='The English help text for this question.', null=True, verbose_name='Help (en)')),
                ('help_de', models.TextField(blank=True, help_text='The German help text for this question.', null=True, verbose_name='Help (de)')),
                ('text_en', models.TextField(help_text='The English text for this question.', verbose_name='Text (en)')),
                ('text_de', models.TextField(help_text='The German text for this question.', verbose_name='Text (de)')),
                ('widget_type', models.CharField(choices=[('text', 'Text'), ('textarea', 'Textarea'), ('yesno', 'Yes/No'), ('checkbox', 'Checkboxes'), ('radio', 'Radio buttons'), ('select', 'Select drop-down'), ('range', 'Range slider'), ('date', 'Date picker')], help_text='Type of widget for this question.', max_length=12, verbose_name='Widget type')),
                ('value_type', models.CharField(choices=[(b'text', 'Text'), (b'url', 'URL'), (b'integer', 'Integer'), (b'float', 'Float'), (b'boolean', 'Boolean'), (b'datetime', 'Datetime'), (b'option', 'Option')], help_text='Type of value for this question.', max_length=8, verbose_name='Value type')),
                ('unit', models.CharField(blank=True, help_text='Unit for this question.', max_length=64, verbose_name='Unit')),
                ('attribute_entity', models.ForeignKey(blank=True, help_text='The attribute/entity this question belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.AttributeEntity', verbose_name='Attribute entity')),
                ('conditions', models.ManyToManyField(blank=True, help_text='List of conditions evaluated for this question.', to='conditions.Condition', verbose_name='Conditions')),
                ('optionsets', models.ManyToManyField(blank=True, help_text='Option sets for this question.', to='options.OptionSet', verbose_name='Option sets')),
            ],
            options={
                'ordering': ('questionset', 'order'),
                'verbose_name': 'Question',
                'verbose_name_plural': 'Questions',
                'permissions': (('view_question', 'Can view Question'),),
            },
            bases=(models.Model, rdmo.core.models.TranslationMixin),
        ),
        migrations.CreateModel(
            name='QuestionSet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(editable=False, verbose_name='created')),
                ('updated', models.DateTimeField(editable=False, verbose_name='updated')),
                ('uri', models.URLField(blank=True, help_text='The Uniform Resource Identifier of this questionset (auto-generated).', max_length=640, null=True, verbose_name='URI')),
                ('uri_prefix', models.URLField(blank=True, help_text='The prefix for the URI of this questionset.', max_length=256, null=True, verbose_name='URI Prefix')),
                ('key', models.SlugField(blank=True, help_text='The internal identifier of this questionset.', max_length=128, null=True, verbose_name='Key')),
                ('path', models.CharField(blank=True, help_text='The path part of the URI of this questionset (auto-generated).', max_length=512, null=True, verbose_name='Path')),
                ('comment', models.TextField(blank=True, help_text='Additional internal information about this questionset.', null=True, verbose_name='Comment')),
                ('is_collection', models.BooleanField(default=False, help_text='Designates whether this questionset is a collection.', verbose_name='is collection')),
                ('order', models.IntegerField(default=0, help_text='The position of this questionset in lists.', verbose_name='Order')),
                ('help_en', models.TextField(blank=True, help_text='The English help text for this questionset.', null=True, verbose_name='Help (en)')),
                ('help_de', models.TextField(blank=True, help_text='The German help text for this questionset.', null=True, verbose_name='Help (de)')),
                ('attribute_entity', models.ForeignKey(blank=True, help_text='The attribute/entity this questionset belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.AttributeEntity', verbose_name='Attribute entity')),
                ('conditions', models.ManyToManyField(blank=True, help_text='List of conditions evaluated for this questionset.', to='conditions.Condition', verbose_name='Conditions')),
                ('subsection', models.ForeignKey(help_text='The subsection this questionset belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='questionsets', to='questions.Subsection', verbose_name='Subsection')),
            ],
            options={
                'ordering': ('subsection', 'order'),
                'verbose_name': 'Question set',
                'verbose_name_plural': 'Question set',
                'permissions': (('view_questionset', 'Can view Question set'),),
            },
            bases=(models.Model, rdmo.core.models.TranslationMixin),
        ),
        migrations.AddField(
            model_name='questionitem',
            name='questionset',
            field=models.ForeignKey(help_text='The question set this question belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='questions', to='questions.QuestionSet', verbose_name='Questionset'),
        ),
    ]



from django.urls import include, path

from rest_framework import routers

from ..viewsets import (CatalogViewSet, QuestionSetViewSet, QuestionViewSet,
                        SectionViewSet, ValueTypeViewSet, WidgetTypeViewSet)

app_name = 'v1-questions'

router = routers.DefaultRouter()
router.register(r'catalogs', CatalogViewSet, basename='catalog')
router.register(r'sections', SectionViewSet, basename='section')
router.register(r'questionsets', QuestionSetViewSet, basename='questionset')
router.register(r'questions', QuestionViewSet, basename='question')
router.register(r'widgettypes', WidgetTypeViewSet, basename='widgettype')
router.register(r'valuetypes', ValueTypeViewSet, basename='valuetype')

urlpatterns = [
    path('', include(router.urls)),
]



import logging

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.auth.mixins import \
    PermissionRequiredMixin as DjangoPermissionRequiredMixin
from django.contrib.auth.views import redirect_to_login
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse
from django.utils import translation
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import ensure_csrf_cookie
from django.views.generic.base import View
from rest_framework import mixins, viewsets
from rest_framework.response import Response
from rules.contrib.views import \
    PermissionRequiredMixin as RulesPermissionRequiredMixin

from .serializers import ChoicesSerializer
from .utils import get_next, get_referer, get_referer_path_info

logger = logging.getLogger(__name__)


def home(request):
    if request.user.is_authenticated:
        return HttpResponseRedirect(reverse('projects'))
    else:
        if settings.SHIBBOLETH:
            return render(request, 'core/home.html')
        elif settings.ACCOUNT or settings.SOCIALACCOUNT:
            from allauth.account.forms import LoginForm
            return render(request, 'core/home.html', {
                'form': LoginForm(),
                'signup_url': reverse("account_signup")
            })
        else:
            from django.contrib.auth.forms import AuthenticationForm
            return render(request, 'core/home.html', {'form': AuthenticationForm()})


@login_required
def about(request):
    return render(request, 'core/about.html')


def i18n_switcher(request, language):
    referer = get_referer(request, default='/')

    # set the new language
    translation.activate(language)

    # get the response, set the cookie and return
    response = HttpResponseRedirect(referer)
    response.set_cookie(settings.LANGUAGE_COOKIE_NAME, language)
    return response


def bad_request(request, exception):
    return render(request, 'core/400.html', status=400)


def forbidden(request, exception):
    return render(request, 'core/403.html', status=403)


def not_found(request, exception):
    return render(request, 'core/404.html', status=404)


def internal_server_error(request):
    return render(request, 'core/500.html', status=500)


class CSRFViewMixin(View):

    @method_decorator(ensure_csrf_cookie)
    def get(self, request, *args, **kwargs):
        return super().get(self, request, *args, **kwargs)


class RedirectViewMixin(View):

    def post(self, request, *args, **kwargs):
        if 'cancel' in request.POST:
            return HttpResponseRedirect(get_next(request))
        else:
            return super(RedirectViewMixin, self).post(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context_data = super(RedirectViewMixin, self).get_context_data(**kwargs)
        if 'next' in self.request.GET:
            context_data['next'] = self.request.GET['next']
        else:
            context_data['next'] = get_referer_path_info(self.request)
        return context_data

    def get_success_url(self):
        if 'next' in self.request.GET:
            return self.request.GET['next']
        else:
            return super(RedirectViewMixin, self).get_success_url()


class PermissionRedirectMixin(object):

    def handle_no_permission(self):
        if self.request.user.is_authenticated:
            raise PermissionDenied(self.get_permission_denied_message())

        return redirect_to_login(self.request.get_full_path(), self.get_login_url(), self.get_redirect_field_name())


class ModelPermissionMixin(PermissionRedirectMixin, DjangoPermissionRequiredMixin, object):
    pass


class ObjectPermissionMixin(PermissionRedirectMixin, RulesPermissionRequiredMixin, object):
    pass


class ChoicesViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    serializer_class = ChoicesSerializer


class SettingsViewSet(viewsets.GenericViewSet):

    def list(self, request, *args, **kwargs):
        return Response({
            'default_uri_prefix': settings.DEFAULT_URI_PREFIX
        })



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-27 15:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('views', '0004_refactoring'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='view',
            options={'ordering': ('key',), 'verbose_name': 'View', 'verbose_name_plural': 'Views'},
        ),
        migrations.AlterField(
            model_name='view',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this view.', null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='view',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this view. The URI will be generated from this key.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='view',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this view (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
        migrations.AlterField(
            model_name='view',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this view.', max_length=256, null=True, verbose_name='URI Prefix'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-03 11:30
from __future__ import unicode_literals

from django.db import migrations, models


def migrate_options(apps, schema_editor):

    Option = apps.get_model('options', 'Option')

    for option in Option.objects.all():
        option.path = option.optionset.key + '/' + option.key
        option.save()


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0008_option_path'),
    ]

    operations = [
        migrations.RunPython(migrate_options),
    ]



import os

import pytest
from django.urls import reverse

from rdmo.core.xml import flat_xml_to_elements, read_xml_file

users = (
    ('editor', 'editor'),
    ('reviewer', 'reviewer'),
    ('user', 'user'),
    ('api', 'api'),
    ('anonymous', None),
)

files = (
    'conditions.xml',
    'domain.xml',
    'options.xml',
    'questions.xml',
    'tasks.xml',
    'views.xml'
)

status_map = {
    'upload_get': {
        'editor': 302, 'reviewer': 302, 'api': 302, 'user': 302, 'anonymous': 302
    },
    'upload_post': {
        'editor': 200, 'reviewer': 403, 'api': 200, 'user': 403, 'anonymous': 302
    },
    'upload_post_empty': {
        'editor': 302, 'reviewer': 302, 'api': 302, 'user': 302, 'anonymous': 302
    },
    'upload_post_error': {
        'editor': 400, 'reviewer': 400, 'api': 400, 'user': 400, 'anonymous': 302
    },
    'import_get': {
        'editor': 302, 'reviewer': 302, 'api': 302, 'user': 302, 'anonymous': 302
    },
    'import_post': {
        'editor': 200, 'reviewer': 403, 'api': 200, 'user': 403, 'anonymous': 302
    },
    'import_post_empty': {
        'editor': 302, 'reviewer': 403, 'api': 302, 'user': 403, 'anonymous': 302
    },
    'import_post_error': {
        'editor': 400, 'reviewer': 400, 'api': 400, 'user': 400, 'anonymous': 302
    }
}


@pytest.mark.parametrize('username,password', users)
def test_upload_get(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('upload')
    response = client.get(url)

    assert response.status_code == status_map['upload_get'][username], response.content

    if not password:
        assert response.url.startswith('/account/login/'), response.content


@pytest.mark.parametrize('file_name', files)
@pytest.mark.parametrize('username,password', users)
def test_upload_post_conditions(db, settings, client, username, password, file_name):
    client.login(username=username, password=password)

    xml_file = os.path.join(settings.BASE_DIR, 'xml', file_name)

    url = reverse('upload')
    with open(xml_file, encoding='utf8') as f:
        response = client.post(url, {'uploaded_file': f})

    assert response.status_code == status_map['upload_post'][username]

    if not password:
        assert response.url.startswith('/account/login/'), response.content


@pytest.mark.parametrize('username,password', users)
def test_upload_post_empty(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('upload')
    response = client.post(url)

    assert response.status_code == status_map['upload_post_empty'][username]

    if not password:
        assert response.url.startswith('/account/login/'), response.content


@pytest.mark.parametrize('username,password', users)
def test_upload_post_error(db, settings, client, username, password):
    client.login(username=username, password=password)

    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'error.xml')

    url = reverse('upload')
    with open(xml_file, encoding='utf8') as f:
        response = client.post(url, {'uploaded_file': f})

    assert response.status_code == status_map['upload_post_error'][username]

    if not password:
        assert response.url.startswith('/account/login/'), response.content


@pytest.mark.parametrize('username,password', users)
def test_import_get(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('import')
    response = client.get(url)

    assert response.status_code == status_map['import_get'][username], response.content

    if not password:
        assert response.url.startswith('/account/login/'), response.content


@pytest.mark.parametrize('file_name', files)
@pytest.mark.parametrize('username,password', users)
def test_import_post(db, settings, client, username, password, file_name):
    client.login(username=username, password=password)

    xml_file = os.path.join(settings.BASE_DIR, 'xml', file_name)

    session = client.session
    session['import_file_name'] = file_name
    session['import_tmpfile_name'] = xml_file
    session.save()

    root = read_xml_file(xml_file)
    elements = flat_xml_to_elements(root)
    checked = [element.get('uri') for element in elements]
    data = {uri: ['on'] for uri in checked}

    url = reverse('import')
    response = client.post(url, data)

    assert response.status_code == status_map['import_post'][username]

    if not password:
        assert response.url.startswith('/account/login/'), response.content


@pytest.mark.parametrize('file_name', files)
@pytest.mark.parametrize('username,password', users)
def test_import_post_empty(db, settings, client, username, password, file_name):
    client.login(username=username, password=password)

    xml_file = os.path.join(settings.BASE_DIR, 'xml', file_name)

    session = client.session
    session['import_file_name'] = file_name
    session['import_tmpfile_name'] = xml_file
    session.save()

    url = reverse('import')
    response = client.post(url)

    assert response.status_code == status_map['import_post_empty'][username]

    if not password:
        assert response.url.startswith('/account/login/'), response.content


@pytest.mark.parametrize('username,password', users)
def test_import_post_error(db, settings, client, username, password):
    client.login(username=username, password=password)

    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'error.xml')

    session = client.session
    session['import_file_name'] = 'error.xml'
    session['import_tmpfile_name'] = xml_file
    session.save()

    url = reverse('import')
    response = client.post(url)

    assert response.status_code == status_map['import_post_error'][username]

    if not password:
        assert response.url.startswith('/account/login/'), response.content



# Generated by Django 2.2.16 on 2020-12-09 18:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0043_meta'),
    ]

    operations = [
        migrations.AlterField(
            model_name='value',
            name='option',
            field=models.ForeignKey(blank=True, help_text='The option stored for this value.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='values', to='options.Option', verbose_name='Option'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0001_initial_after_reset'),
    ]

    operations = [
        migrations.AlterField(
            model_name='option',
            name='additional_input',
            field=models.BooleanField(default=False),
        ),
    ]



import logging

from django.contrib.sites.models import Site

from rdmo.conditions.models import Condition
from rdmo.core.imports import (fetch_parents, get_foreign_field,
                               get_m2m_instances, set_common_fields,
                               set_lang_field, validate_instance)
from rdmo.domain.models import Attribute
from rdmo.options.models import Option, OptionSet

from .models import Catalog, Question, QuestionSet, Section
from .validators import (CatalogLockedValidator, CatalogUniqueURIValidator,
                         QuestionLockedValidator, QuestionSetLockedValidator,
                         QuestionSetUniqueURIValidator,
                         QuestionUniqueURIValidator, SectionLockedValidator,
                         SectionUniqueURIValidator)
from .utils import get_widget_types

logger = logging.getLogger(__name__)


def import_catalog(element, save=False):
    try:
        catalog = Catalog.objects.get(uri=element.get('uri'))
    except Catalog.DoesNotExist:
        catalog = Catalog()

    set_common_fields(catalog, element)

    catalog.order = element.get('order') or 0

    set_lang_field(catalog, 'title', element)
    set_lang_field(catalog, 'help', element)

    if save and validate_instance(catalog, CatalogLockedValidator, CatalogUniqueURIValidator):
        if catalog.id:
            logger.info('Catalog created with uri %s.', element.get('uri'))
        else:
            logger.info('Catalog %s updated.', element.get('uri'))

        catalog.save()
        catalog.sites.add(Site.objects.get_current())
        catalog.imported = True

    return catalog


def import_section(element, catalog_uri=False, save=False):
    try:
        if catalog_uri is False:
            section = Section.objects.get(uri=element.get('uri'))
        else:
            section = Section.objects.get(key=element.get('key'), catalog__uri=catalog_uri)

    except Section.DoesNotExist:
        section = Section()

    set_common_fields(section, element)

    section.catalog = get_foreign_field(section, catalog_uri or element.get('catalog'), Catalog)
    section.order = element.get('order') or 0

    set_lang_field(section, 'title', element)

    if save and validate_instance(section, SectionLockedValidator, SectionUniqueURIValidator):
        if section.id:
            logger.info('Section created with uri %s.', element.get('uri'))
        else:
            logger.info('Section %s updated.', element.get('uri'))

        section.save()
        section.imported = True

    return section


def import_questionset(element, section_uri=False, questionset_uri=False, save=False):
    try:
        if section_uri is False:
            questionset = QuestionSet.objects.get(uri=element.get('uri'))
        else:
            if questionset_uri is False:
                questionset = QuestionSet.objects.get(key=element.get('key'), section__uri=section_uri)
            else:
                questionset = QuestionSet.objects.get(key=element.get('key'), section__uri=section_uri, questionset__uri=questionset_uri)

    except QuestionSet.DoesNotExist:
        questionset = QuestionSet()

    set_common_fields(questionset, element)

    questionset.section = get_foreign_field(questionset, section_uri or element.get('section'), Section)
    questionset.questionset = get_foreign_field(questionset, questionset_uri or element.get('questionset'), QuestionSet)
    questionset.attribute = get_foreign_field(questionset, element.get('attribute'), Attribute)
    questionset.is_collection = element.get('is_collection') or False
    questionset.order = element.get('order') or 0

    set_lang_field(questionset, 'title', element)
    set_lang_field(questionset, 'help', element)
    set_lang_field(questionset, 'verbose_name', element)
    set_lang_field(questionset, 'verbose_name_plural', element)

    conditions = get_m2m_instances(questionset, element.get('conditions'), Condition)

    if save and validate_instance(questionset, QuestionSetLockedValidator, QuestionSetUniqueURIValidator):
        if questionset.id:
            logger.info('QuestionSet created with uri %s.', element.get('uri'))
        else:
            logger.info('QuestionSet %s updated.', element.get('uri'))

        questionset.save()
        questionset.conditions.set(conditions)
        questionset.imported = True

    return questionset


def import_question(element, questionset_uri=False, save=False):
    try:
        if questionset_uri is False:
            question = Question.objects.get(uri=element.get('uri'))
        else:
            question = Question.objects.get(key=element.get('key'), questionset__uri=questionset_uri)
    except Question.DoesNotExist:
        question = Question()

    set_common_fields(question, element)

    question.questionset = get_foreign_field(question, questionset_uri or element.get('questionset'), QuestionSet)
    question.attribute = get_foreign_field(question, element.get('attribute'), Attribute)
    question.is_collection = element.get('is_collection') or False
    question.is_optional = element.get('is_optional') or False
    question.order = element.get('order') or 0

    set_lang_field(question, 'text', element)
    set_lang_field(question, 'help', element)
    set_lang_field(question, 'default_text', element)
    set_lang_field(question, 'verbose_name', element)
    set_lang_field(question, 'verbose_name_plural', element)

    question.default_option = get_foreign_field(question, element.get('default_option'), Option)
    question.default_external_id = element.get('default_external_id') or ''

    if element.get('widget_type') in get_widget_types():
        question.widget_type = element.get('widget_type')
    else:
        question.widget_type = 'text'
    question.value_type = element.get('value_type') or ''
    question.maximum = element.get('maximum')
    question.minimum = element.get('minimum')
    question.step = element.get('step')
    question.unit = element.get('unit') or ''
    question.width = element.get('width')

    conditions = get_m2m_instances(question, element.get('conditions'), Condition)
    optionsets = get_m2m_instances(question, element.get('optionsets'), OptionSet)

    if save and validate_instance(question, QuestionLockedValidator, QuestionUniqueURIValidator):
        if question.id:
            logger.info('Question created with uri %s.', element.get('uri'))
        else:
            logger.info('Question %s updated.', element.get('uri'))

        question.save()
        question.conditions.set(conditions)
        question.optionsets.set(optionsets)
        question.imported = True

    return question


def fetch_section_parents(instances):
    return fetch_parents(Catalog, instances)


def fetch_questionset_parents(instances):
    return fetch_parents(Section, instances)


def fetch_question_parents(instances):
    return fetch_parents(QuestionSet, instances)



# Generated by Django 2.2.18 on 2021-02-09 12:09

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0047_continuation'),
    ]

    operations = [
        migrations.AlterField(
            model_name='membership',
            name='project',
            field=models.ForeignKey(help_text='The project for this membership.', on_delete=django.db.models.deletion.CASCADE, related_name='memberships', to='projects.Project', verbose_name='Project'),
        ),
        migrations.AlterField(
            model_name='membership',
            name='user',
            field=models.ForeignKey(help_text='The user for this membership.', on_delete=django.db.models.deletion.CASCADE, related_name='memberships', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AlterField(
            model_name='project',
            name='user',
            field=models.ManyToManyField(help_text='The list of users for this project.', related_name='projects', through='projects.Membership', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-13 11:54
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0041_data_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this attribute.', verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this attribute.', max_length=128, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this attribute (auto-generated).', max_length=640, verbose_name='URI'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this attribute.', max_length=256, verbose_name='URI Prefix'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-03 13:27
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0027_meta'),
    ]

    operations = [
        migrations.RenameField(
            model_name='attributeentity',
            old_name='label',
            new_name='path',
        ),
    ]



import pytest
from django.urls import reverse

from ..models import Overlay

users = (
    ('user', 'user'),
    ('anonymous', None)
)

urlnames = {
    'current': 'v1-overlays:overlay-current',
    'next': 'v1-overlays:overlay-next',
    'dismiss': 'v1-overlays:overlay-dismiss'
}

url_names = [
    'projects'
]


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('url_name', url_names)
def test_current(db, client, username, password, url_name):
    client.login(username=username, password=password)

    url = reverse(urlnames['current'], args=[url_name])
    response = client.post(url)

    if password:
        assert response.status_code == 200
        assert Overlay.objects.get(user__username=username, url_name=url_name).current == response.json().get('overlay')
    else:
        assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('url_name', url_names)
def test_next(db, client, username, password, url_name):
    client.login(username=username, password=password)

    url = reverse(urlnames['next'], args=[url_name])
    response = client.post(url)

    if password:
        assert response.status_code == 200
        assert response.json().get('overlay') == Overlay.objects.get(
            user__username=username, url_name=url_name
        ).current
    else:
        assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('url_name', url_names)
def test_dismiss(db, client, username, password, url_name):
    client.login(username=username, password=password)

    url = reverse(urlnames['dismiss'], args=[url_name])
    response = client.post(url)

    if password:
        assert response.status_code == 200
        assert response.json().get('overlay') == ''
        assert response.json().get('overlay') == Overlay.objects.get(
            user__username=username, url_name=url_name
        ).current
    else:
        assert response.status_code == 401



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Condition
from ..serializers.v1 import ConditionSerializer
from ..validators import ConditionUniqueURIValidator


def test_unique_uri_validator_create(db):
    ConditionUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_unique_uri_validator_create_error(db):
    with pytest.raises(ValidationError):
        ConditionUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': Condition.objects.last().key
        })


def test_unique_uri_validator_update(db):
    condition = Condition.objects.first()

    ConditionUniqueURIValidator(condition)({
        'uri_prefix': condition.uri_prefix,
        'key': condition.key
    })


def test_unique_uri_validator_update_error(db):
    condition = Condition.objects.first()

    with pytest.raises(ValidationError):
        ConditionUniqueURIValidator(condition)({
            'uri_prefix': condition.uri_prefix,
            'key': Condition.objects.last().key
        })


def test_unique_uri_validator_serializer_create(db):
    validator = ConditionUniqueURIValidator()
    validator.set_context(ConditionSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_unique_uri_validator_serializer_create_error(db):
    validator = ConditionUniqueURIValidator()
    validator.set_context(ConditionSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': Condition.objects.last().key
        })


def test_unique_uri_validator_serializer_update(db):
    condition = Condition.objects.first()

    validator = ConditionUniqueURIValidator()
    validator.set_context(ConditionSerializer(instance=condition))

    validator({
        'uri_prefix': condition.uri_prefix,
        'key': condition.key
    })


def test_unique_uri_validator_serializer_update_error(db):
    condition = Condition.objects.first()

    validator = ConditionUniqueURIValidator()
    validator.set_context(ConditionSerializer(instance=condition))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': condition.uri_prefix,
            'key': Condition.objects.last().key
        })
